@@ -1,5 +1,5 @@
 /* g10c.c  -  Wrapper for cipher functions
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,10 +24,12 @@
 #include "mpi.h"
 #include "random.h"
 #include "cipher.h"
-#define _g10lib_INTERNAL 1
-#include "g10lib.h"
 
-const char *g10c_revision_string(int dummy) { return "$Revision: 1.5 $"; }
+
+/* FIXME: The modules should use functions from libgcrypt */
+
+
+const char *g10c_revision_string(int dummy) { return "$Revision: 1.6.2.2 $"; }
 
 MPI
 g10c_generate_secret_prime( unsigned nbits )
