@@ -1,5 +1,5 @@
 /* md.c  -  message digest dispatcher
- *	Copyright (C) 1998,1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -291,7 +291,7 @@ md_reset( MD_HANDLE a )
 {
     struct md_digest_list_s *r;
 
-    a->bufcount = 0;
+    a->bufcount = a->finalized = 0;
     for( r=a->list; r; r = r->next ) {
 	memset( r->context.c, 0, r->contextsize );
 	(*r->init)( &r->context.c );
@@ -341,12 +341,16 @@ md_final(MD_HANDLE a)
 {
     struct md_digest_list_s *r;
 
+    if( a->finalized )
+	return;
+
     if( a->bufcount )
 	md_write( a, NULL, 0 );
 
     for(r=a->list; r; r = r->next ) {
 	(*r->final)( &r->context.c );
     }
+    a->finalized = 1;
 }
 
 
