@@ -1,5 +1,5 @@
 /* random.c  -	random number generator
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -32,6 +32,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
+#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -43,9 +44,6 @@
 #ifdef HAVE_GETTIMEOFDAY
   #include <sys/times.h>
 #endif
-#ifdef HAVE_CLOCK_GETTIME
-  #include <time.h>
-#endif
 #ifdef HAVE_GETRUSAGE
   #include <sys/resource.h>
 #endif
@@ -141,6 +139,16 @@ initialize(void)
     cipher_modules_constructor();
 }
 
+static void
+burn_stack (int bytes)
+{
+    char buf[128];
+    
+    memset (buf, 0, sizeof buf);
+    bytes -= sizeof buf;
+    if (bytes > 0)
+        burn_stack (bytes);
+}
 
 void
 random_dump_stats()
@@ -271,6 +279,7 @@ mix_pool(byte *pool)
 	rmd160_mixblock( &md, hashbuf);
 	memcpy(p, hashbuf, 20 );
     }
+    burn_stack (200); /* for the rmd160_mixblock() */
 }
 
 
@@ -595,9 +604,11 @@ fast_random_poll()
     #else
     {	struct rusage buf;
         /* QNX/Neutrino does return ENOSYS - so we just ignore it and
-         * add whatever is in buf */
-        if( getrusage( RUSAGE_SELF, &buf ) && errno != ENOSYS )
-	    BUG();
+         * add whatever is in buf.  In a chroot environment it might not
+         * work at all (i.e. because /proc/ is not accessible), so we better 
+         * ognore all error codes and hope for the best
+         */
+        getrusage( RUSAGE_SELF, &buf );
         
 	add_randomness( &buf, sizeof buf, 1 );
 	memset( &buf, 0, sizeof buf );
