@@ -1,5 +1,5 @@
 /* rndlinux.c  -  raw random number for OSes with /dev/random
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -141,7 +141,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	      #endif
 _("\n"
 "Not enough random bytes available.  Please do some other work to give\n"
-"the OS a chance to collect more entropy! (Need %d more bytes)\n"), length );
+"the OS a chance to collect more entropy! (Need %d more bytes)\n"), (int)length );
 	    warn = 1;
 	    continue;
 	}
@@ -178,7 +178,7 @@ _("\n"
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDLINUX ($Revision: 1.9.2.2 $)";
+const char * const gnupgext_version = "RNDLINUX ($Revision: 1.9.2.4 $)";
 
 static struct {
     int class;
