@@ -1,5 +1,5 @@
 /* rndw32.c  -	W32 entropy gatherer
- *	Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
  *	Copyright Peter Gutmann, Matt Thomlinson and Blake Coverett 1996-1999
  *
  * This file is part of GnuPG.
@@ -311,8 +311,8 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 #ifndef IOCTL_DISK_PERFORMANCE
 #define IOCTL_DISK_PERFORMANCE	0x00070020
 #endif
-#ifndef IOCTL_DISK_PERFORMANCE
-#define VER_PLATFORM_WIN32_WINDOWSw 1
+#ifndef VER_PLATFORM_WIN32_WINDOWS
+#define VER_PLATFORM_WIN32_WINDOWS 1
 #endif
 
 typedef struct {
@@ -915,7 +915,7 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDW32 ($Revision: 1.1.4.6 $)";
+const char * const gnupgext_version = "RNDW32 ($Revision: 1.1.4.8 $)";
 
 static struct {
     int class;
