@@ -1,5 +1,5 @@
 /* sha1.c - SHA1 hash function
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * Please see below for more legal information!
  *
@@ -49,7 +49,16 @@ typedef struct {
     int  count;
 } SHA1_CONTEXT;
 
-
+static void
+burn_stack (int bytes)
+{
+    char buf[128];
+    
+    memset (buf, 0, sizeof buf);
+    bytes -= sizeof buf;
+    if (bytes > 0)
+        burn_stack (bytes);
+}
 
 
 void
@@ -214,6 +223,7 @@ sha1_write( SHA1_CONTEXT *hd, byte *inbuf, size_t inlen)
 {
     if( hd->count == 64 ) { /* flush the buffer */
 	transform( hd, hd->buf );
+        burn_stack (88+4*sizeof(void*));
 	hd->count = 0;
 	hd->nblocks++;
     }
@@ -234,6 +244,7 @@ sha1_write( SHA1_CONTEXT *hd, byte *inbuf, size_t inlen)
 	inlen -= 64;
 	inbuf += 64;
     }
+    burn_stack (88+4*sizeof(void*));
     for( ; inlen && hd->count < 64; inlen-- )
 	hd->buf[hd->count++] = *inbuf++;
 }
@@ -254,18 +265,19 @@ sha1_final(SHA1_CONTEXT *hd)
 
     sha1_write(hd, NULL, 0); /* flush */;
 
-    msb = 0;
     t = hd->nblocks;
-    if( (lsb = t << 6) < t ) /* multiply by 64 to make a byte count */
-	msb++;
-    msb += t >> 26;
+    /* multiply by 64 to make a byte count */
+    lsb = t << 6;
+    msb = t >> 26;
+    /* add the count */
     t = lsb;
-    if( (lsb = t + hd->count) < t ) /* add the count */
+    if( (lsb += hd->count) < t )
 	msb++;
+    /* multiply by 8 to make a bit count */
     t = lsb;
-    if( (lsb = t << 3) < t ) /* multiply by 8 to make a bit count */
-	msb++;
-    msb += t >> 29;
+    lsb <<= 3;
+    msb <<= 3;
+    msb |= t >> 29;
 
     if( hd->count < 56 ) { /* enough room */
 	hd->buf[hd->count++] = 0x80; /* pad */
@@ -289,6 +301,7 @@ sha1_final(SHA1_CONTEXT *hd)
     hd->buf[62] = lsb >>  8;
     hd->buf[63] = lsb	   ;
     transform( hd, hd->buf );
+    burn_stack (88+4*sizeof(void*));
 
     p = hd->buf;
   #ifdef BIG_ENDIAN_HOST
@@ -350,7 +363,7 @@ sha1_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "SHA1 ($Revision: 1.12.2.3 $)";
+const char * const gnupgext_version = "SHA1 ($Revision: 1.12.2.6 $)";
 
 static struct {
     int class;
