@@ -1,5 +1,5 @@
 /* tiger.c  -  The TIGER hash function
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -630,6 +630,19 @@ print_data( const char *text, u64 a, u64 b, u64 c,
 }
 
 
+static void
+burn_stack (int bytes)
+{
+    char buf[256];
+    
+    memset (buf, 0, sizeof buf);
+    bytes -= sizeof buf;
+    if (bytes > 0)
+        burn_stack (bytes);
+}
+
+
+
 static void
 tiger_init( TIGER_CONTEXT *hd )
 {
@@ -768,6 +781,7 @@ tiger_write( TIGER_CONTEXT *hd, byte *inbuf, size_t inlen)
 {
     if( hd->count == 64 ) { /* flush the buffer */
 	transform( hd, hd->buf );
+        burn_stack (21*8+11*sizeof(void*));
 	hd->count = 0;
 	hd->nblocks++;
     }
@@ -788,6 +802,7 @@ tiger_write( TIGER_CONTEXT *hd, byte *inbuf, size_t inlen)
 	inlen -= 64;
 	inbuf += 64;
     }
+    burn_stack (21*8+11*sizeof(void*));
     for( ; inlen && hd->count < 64; inlen-- )
 	hd->buf[hd->count++] = *inbuf++;
 }
@@ -805,18 +820,19 @@ tiger_final( TIGER_CONTEXT *hd )
 
     tiger_write(hd, NULL, 0); /* flush */;
 
-    msb = 0;
     t = hd->nblocks;
-    if( (lsb = t << 6) < t ) /* multiply by 64 to make a byte count */
-	msb++;
-    msb += t >> 26;
+    /* multiply by 64 to make a byte count */
+    lsb = t << 6;
+    msb = t >> 26;
+    /* add the count */
     t = lsb;
-    if( (lsb = t + hd->count) < t ) /* add the count */
+    if( (lsb += hd->count) < t )
 	msb++;
+    /* multiply by 8 to make a bit count */
     t = lsb;
-    if( (lsb = t << 3) < t ) /* multiply by 8 to make a bit count */
-	msb++;
-    msb += t >> 29;
+    lsb <<= 3;
+    msb <<= 3;
+    msb |= t >> 29;
 
     if( hd->count < 56 ) { /* enough room */
 	hd->buf[hd->count++] = 0x01; /* pad */
@@ -840,6 +856,7 @@ tiger_final( TIGER_CONTEXT *hd )
     hd->buf[62] = msb >> 16;
     hd->buf[63] = msb >> 24;
     transform( hd, hd->buf );
+    burn_stack (21*8+11*sizeof(void*));
 
     p = hd->buf;
   #ifdef BIG_ENDIAN_HOST
@@ -904,7 +921,7 @@ tiger_get_info( int algo, size_t *contextsize,
     *(void  (**)(TIGER_CONTEXT *))r_final		 = tiger_final;
     *(byte *(**)(TIGER_CONTEXT *))r_read		 = tiger_read;
 
-    return "TIGER";
+    return "TIGER192";
 }
 
 
@@ -912,7 +929,7 @@ tiger_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "TIGER ($Revision: 1.10.2.1 $)";
+const char * const gnupgext_version = "TIGER ($Revision: 1.10.2.5 $)";
 
 static struct {
     int class;
