@@ -1,5 +1,5 @@
-/* verify.c - verify signed data
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+/* decrypt.c - verify signed data
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -71,7 +71,7 @@ decrypt_message( const char *filename )
 	no_out = 1;
 	opt.outfile = "-";
     }
-    rc = proc_encryption_packets( fp );
+    rc = proc_encryption_packets( NULL, fp );
     if( no_out )
        opt.outfile = NULL;
     iobuf_close(fp);
