@@ -1,5 +1,5 @@
 /* encode.c - encode data
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -36,7 +36,7 @@
 #include "filter.h"
 #include "trustdb.h"
 #include "i18n.h"
-
+#include "status.h"
 
 static int encode_simple( const char *filename, int mode );
 static int write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out );
@@ -221,8 +221,11 @@ encode_simple( const char *filename, int mode )
     iobuf_close(inp);
     if (rc)
 	iobuf_cancel(out);
-    else
+    else {
 	iobuf_close(out); /* fixme: check returncode */
+        if (mode)
+            write_status( STATUS_END_ENCRYPTION );
+    }
     if (pt)
 	pt->buf = NULL;
     free_packet(&pkt);
@@ -288,7 +291,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     }
   #endif
     /* create a session key */
-    cfx.dek = m_alloc_secure( sizeof *cfx.dek );
+    cfx.dek = m_alloc_secure_clear (sizeof *cfx.dek);
     if( !opt.def_cipher_algo ) { /* try to get it from the prefs */
 	cfx.dek->algo = select_algo_from_prefs( pk_list, PREFTYPE_SYM );
 	if( cfx.dek->algo == -1 )
@@ -296,6 +299,8 @@ encode_crypt( const char *filename, STRLIST remusr )
     }
     else
 	cfx.dek->algo = opt.def_cipher_algo;
+    cfx.dek->use_mdc = select_mdc_from_pklist (pk_list);
+
     make_session_key( cfx.dek );
     if( DBG_CIPHER )
 	log_hexdump("DEK is: ", cfx.dek->key, cfx.dek->keylen );
@@ -323,7 +328,7 @@ encode_crypt( const char *filename, STRLIST remusr )
 	if( !(filesize = iobuf_get_filelength(inp)) )
 	    log_info(_("%s: WARNING: empty file\n"), filename );
         /* we can't yet encode the length of very large files,
-         * so we switch to partial lengthn encoding in this case */
+         * so we switch to partial length encoding in this case */
         if ( filesize >= IOBUF_FILELENGTH_LIMIT )
             filesize = 0;
     }
@@ -348,7 +353,7 @@ encode_crypt( const char *filename, STRLIST remusr )
 
     /* register the compress filter */
     if( do_compress ) {
-	int compr_algo = select_algo_from_prefs( pk_list, PREFTYPE_COMPR );
+	int compr_algo = select_algo_from_prefs( pk_list, PREFTYPE_ZIP );
 	if( !compr_algo )
 	    ; /* don't use compression */
 	else {
@@ -381,8 +386,10 @@ encode_crypt( const char *filename, STRLIST remusr )
     iobuf_close(inp);
     if( rc )
 	iobuf_cancel(out);
-    else
+    else {
 	iobuf_close(out); /* fixme: check returncode */
+        write_status( STATUS_END_ENCRYPTION );
+    }
     if( pt )
 	pt->buf = NULL;
     free_packet(&pkt);
@@ -410,16 +417,22 @@ encrypt_filter( void *opaque, int control,
     }
     else if( control == IOBUFCTRL_FLUSH ) { /* encrypt */
 	if( !efx->header_okay ) {
-	    efx->cfx.dek = m_alloc_secure( sizeof *efx->cfx.dek );
+	    efx->cfx.dek = m_alloc_secure_clear( sizeof *efx->cfx.dek );
 
 	    if( !opt.def_cipher_algo  ) { /* try to get it from the prefs */
 		efx->cfx.dek->algo =
 			  select_algo_from_prefs( efx->pk_list, PREFTYPE_SYM );
-		if( efx->cfx.dek->algo == -1 )
+		if( efx->cfx.dek->algo == -1 ) {
+                    /* because 3DES is implicitly in the prefs, this can only
+                     * happen if we do not have any public keys in the list */
 		    efx->cfx.dek->algo = DEFAULT_CIPHER_ALGO;
+                }
 	    }
 	    else
 		efx->cfx.dek->algo = opt.def_cipher_algo;
+
+            efx->cfx.dek->use_mdc = select_mdc_from_pklist (efx->pk_list);
+
 	    make_session_key( efx->cfx.dek );
 	    if( DBG_CIPHER )
 		log_hexdump("DEK is: ",
