@@ -1,5 +1,5 @@
 /* encr-data.c -  process an encrypted data packet
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -93,6 +93,10 @@ decrypt_data( void *procctx, PKT_encrypted *ed, DEK *dek )
 	log_error("key setup failed: %s\n", g10_errstr(rc) );
 	goto leave;
     }
+    if (!ed->buf) {
+        log_error(_("problem handling encrypted packet\n"));
+        goto leave;
+    }
 
     cipher_setiv( dfx.cipher_hd, NULL, 0 );
 
