@@ -1,5 +1,5 @@
-/* hkp.h  -  Horrowitz Keyserver Protocol
- *	Copyright (C) 1999 Free Software Foundation, Inc.
+/* hkp.h  -  Horowitz Keyserver Protocol
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -22,9 +22,10 @@
 #define G10_HKP_H 1
 
 
-int hkp_ask_import( u32 *keyid );
+int hkp_ask_import( u32 *keyid, void *stats_handle);
 int hkp_import( STRLIST users );
 int hkp_export( STRLIST users );
+int hkp_refresh_keys (void);
 
 
 #endif /*G10_HKP_H*/
