@@ -1,5 +1,5 @@
 /* keygen.c - generate a key pair
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -35,11 +35,16 @@
 #include "status.h"
 #include "i18n.h"
 
+#define MAX_PREFS 30 
+
+
 enum para_name {
   pKEYTYPE,
   pKEYLENGTH,
+  pKEYUSAGE,
   pSUBKEYTYPE,
   pSUBKEYLENGTH,
+  pSUBKEYUSAGE,
   pNAMEREAL,
   pNAMEEMAIL,
   pNAMECOMMENT,
@@ -57,10 +62,11 @@ struct para_data_s {
     int lnr;
     enum para_name key;
     union {
-       DEK *dek;
-       STRING2KEY *s2k;
-       u32 expire;
-       char value[1];
+        DEK *dek;
+        STRING2KEY *s2k;
+        u32 expire;
+        unsigned int usage; 
+        char value[1];
     } u;
 };
 
@@ -83,6 +89,21 @@ struct output_control_s {
 };
 
 
+struct opaque_data_usage_and_pk {
+    unsigned int usage;
+    PKT_public_key *pk;
+};
+
+
+static int prefs_initialized = 0;
+static byte sym_prefs[MAX_PREFS];
+static int nsym_prefs;
+static byte hash_prefs[MAX_PREFS];
+static int nhash_prefs;
+static byte zip_prefs[MAX_PREFS];
+static int nzip_prefs;
+
+
 static void do_generate_keypair( struct para_data_s *para,
 				 struct output_control_s *outctrl );
 static int  write_keyblock( IOBUF out, KBNODE node );
@@ -97,10 +118,26 @@ write_uid( KBNODE root, const char *s )
     pkt->pkttype = PKT_USER_ID;
     pkt->pkt.user_id = m_alloc_clear( sizeof *pkt->pkt.user_id + n - 1 );
     pkt->pkt.user_id->len = n;
+    pkt->pkt.user_id->ref = 1;
     strcpy(pkt->pkt.user_id->name, s);
     add_kbnode( root, new_kbnode( pkt ) );
 }
 
+static void
+do_add_key_flags (PKT_signature *sig, unsigned int usage)
+{
+    byte buf[1];
+
+    if (!usage) 
+        return;
+
+    buf[0] = 0;
+    if (usage & PUBKEY_USAGE_SIG)
+        buf[0] |= 0x01 | 0x02;
+    if (usage & PUBKEY_USAGE_ENC)
+        buf[0] |= 0x04 | 0x08;
+    build_sig_subpkt (sig, SIGSUBPKT_KEY_FLAGS, buf, 1);
+}
 
 
 int
@@ -123,6 +160,175 @@ keygen_add_key_expire( PKT_signature *sig, void *opaque )
     return 0;
 }
 
+static int
+keygen_add_key_flags_and_expire (PKT_signature *sig, void *opaque)
+{
+    struct opaque_data_usage_and_pk *oduap = opaque;
+
+    do_add_key_flags (sig, oduap->usage);
+    return keygen_add_key_expire (sig, oduap->pk);
+}
+
+static int
+set_one_pref (ulong val, int type, int (*cf)(int), byte *buf, int *nbuf)
+{
+    int i;
+
+    if (!val || val > 127 || cf (val)) {
+        log_info (_("preference %c%lu is not valid\n"), type, val);
+        return -1;
+    }
+    for (i=0; i < *nbuf; i++ ) {
+        if (buf[i] == val) {
+            log_info (_("preference %c%lu duplicated\n"), type, val);
+            return -1;
+        }
+    }
+    if (*nbuf >= MAX_PREFS) {
+        log_info (_("too many `%c' preferences\n"), type);
+        return -1;
+    }
+    buf[(*nbuf)++] = val;
+    return 0;
+}
+
+static int 
+check_zip_algo (int algo)
+{
+    return algo < 0 || algo > 2;
+}
+
+/*
+ * Parse the supplied string and use it to set the standard preferences.
+ * The String is expected to be in a forma like the one printed by "prefs",
+ * something like:  "S10 S3 H3 H2 Z2 Z1".  Use NULL to set the default
+ * preferences.
+ * Returns: 0 = okay
+ */
+int
+keygen_set_std_prefs (const char *string)
+{
+    byte sym[MAX_PREFS], hash[MAX_PREFS], zip[MAX_PREFS];
+    int  nsym=0, nhash=0, nzip=0;
+    ulong val;
+    const char *s, *s2;
+    int rc = 0;
+
+    if (!string || !ascii_strcasecmp (string, "default"))
+        string = "S7 S10 S3 S4 H3 H2 Z2 Z1";
+    else if (!ascii_strcasecmp (string, "none"))
+        string = "";
+
+    for (s=string; *s; s = s2) {
+        if ((*s=='s' || *s == 'S') && isdigit(s[1]) ) {
+            val = strtoul (++s, (char**)&s2, 10);
+            if (set_one_pref (val, 'S', check_cipher_algo, sym, &nsym))
+                rc = -1;
+        }
+        else if ((*s=='h' || *s == 'H') && isdigit(s[1]) ) {
+            val = strtoul (++s, (char**)&s2, 10);
+            if (set_one_pref (val, 'H', check_digest_algo, hash, &nhash))
+                rc = -1;
+        }
+        else if ((*s=='z' || *s == 'Z') && isdigit(s[1]) ) {
+            val = strtoul (++s, (char**)&s2, 10);
+            if (set_one_pref (val, 'Z', check_zip_algo, zip, &nzip))
+                rc = -1;
+        }
+        else if (isspace (*s))
+            s2 = s+1;
+        else {
+            log_info (_("invalid character in string\n"));
+            return -1;
+        }
+    }
+
+    if (!rc) {
+        memcpy (sym_prefs,  sym,  (nsym_prefs=nsym));
+        memcpy (hash_prefs, hash, (nhash_prefs=nhash));
+        memcpy (zip_prefs,  zip,  (nzip_prefs=nzip));
+        prefs_initialized = 1;
+    }
+    return rc;
+}
+
+
+/* 
+ * Return a printable list of preferences.  Caller must free.
+ */
+char *
+keygen_get_std_prefs ()
+{
+    char *buf;
+    int i;
+
+    if (!prefs_initialized)
+        keygen_set_std_prefs (NULL);
+
+    buf = m_alloc ( MAX_PREFS*3*5 + 1);
+    *buf = 0;
+    for (i=0; i < nsym_prefs; i++ )
+        sprintf (buf+strlen(buf), "S%d ", sym_prefs[i]);
+    for (i=0; i < nhash_prefs; i++ )
+        sprintf (buf+strlen(buf), "H%d ", hash_prefs[i]);
+    for (i=0; i < nzip_prefs; i++ )
+        sprintf (buf+strlen(buf), "Z%d ", zip_prefs[i]);
+    
+    if (*buf) /* trim the trailing space */
+        buf[strlen(buf)-1] = 0; 
+    return buf;
+}
+
+
+static void
+add_feature_mdc (PKT_signature *sig)
+{
+    const byte *s;
+    size_t n;
+    char *buf;
+
+    s = parse_sig_subpkt (sig->hashed, SIGSUBPKT_FEATURES, &n );
+    if (s && n && (s[0] & 0x01))
+        return; /* already set */
+    if (!s || !n) { /* create a new one */
+        n = 1;
+        buf = m_alloc (n);
+    }
+    else {
+        buf = m_alloc (n);
+        memcpy (buf, s, n);
+    }
+    buf[0] |= 0x01; /* MDC feature */
+    build_sig_subpkt (sig, SIGSUBPKT_FEATURES, buf, n);
+    m_free (buf);
+}
+
+
+int
+keygen_upd_std_prefs( PKT_signature *sig, void *opaque )
+{
+    if (!prefs_initialized)
+        keygen_set_std_prefs (NULL);
+
+    if (nsym_prefs) 
+        build_sig_subpkt (sig, SIGSUBPKT_PREF_SYM, sym_prefs, nsym_prefs);
+    else
+        delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_SYM);
+    if (nhash_prefs)
+        build_sig_subpkt (sig, SIGSUBPKT_PREF_HASH, hash_prefs, nhash_prefs);
+    else
+        delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_HASH);
+    if (nzip_prefs)
+        build_sig_subpkt (sig, SIGSUBPKT_PREF_COMPR, zip_prefs, nzip_prefs);
+    else
+        delete_sig_subpkt (sig->hashed, SIGSUBPKT_PREF_COMPR);
+
+    /* Make sure that the MDC feature flag is set */
+    add_feature_mdc (sig);
+
+    return 0;
+}
+
 
 /****************
  * Add preference to the self signature packet.
@@ -131,23 +337,12 @@ keygen_add_key_expire( PKT_signature *sig, void *opaque )
 int
 keygen_add_std_prefs( PKT_signature *sig, void *opaque )
 {
+    PKT_public_key *pk = opaque;
     byte buf[8];
 
+    do_add_key_flags (sig, pk->pubkey_usage);
     keygen_add_key_expire( sig, opaque );
-
-    buf[0] = CIPHER_ALGO_RIJNDAEL;
-    buf[1] = CIPHER_ALGO_TWOFISH;
-    buf[2] = CIPHER_ALGO_CAST5;
-    buf[3] = CIPHER_ALGO_BLOWFISH;
-    build_sig_subpkt( sig, SIGSUBPKT_PREF_SYM, buf, 4 );
-
-    buf[0] = DIGEST_ALGO_RMD160;
-    buf[1] = DIGEST_ALGO_SHA1;
-    build_sig_subpkt( sig, SIGSUBPKT_PREF_HASH, buf, 2 );
-
-    buf[0] = 2;
-    buf[1] = 1;
-    build_sig_subpkt( sig, SIGSUBPKT_PREF_COMPR, buf, 2 );
+    keygen_upd_std_prefs (sig, opaque);
 
     buf[0] = 0x80; /* no modify - It is reasonable that a key holder
 		    * has the possibility to reject signatures from users
@@ -161,9 +356,9 @@ keygen_add_std_prefs( PKT_signature *sig, void *opaque )
 }
 
 
-
 static int
-write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk )
+write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
+               unsigned int usage )
 {
     PACKET *pkt;
     PKT_signature *sig;
@@ -185,10 +380,14 @@ write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk )
     if( !node )
 	BUG();
     pk = node->pkt->pkt.public_key;
+    pk->pubkey_usage = usage;
+    /* we have to cache the key, so that the verification of the signature
+     * creation is able to retrieve the public key */
+    cache_public_key (pk);
 
     /* and make the signature */
-    rc = make_keysig_packet( &sig, pk, uid, NULL, sk, 0x13, 0,
-			     keygen_add_std_prefs, pk );
+    rc = make_keysig_packet( &sig, pk, uid, NULL, sk, 0x13, 0, 0,
+        		     keygen_add_std_prefs, pk );
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
 	return rc;
@@ -202,13 +401,15 @@ write_selfsig( KBNODE root, KBNODE pub_root, PKT_secret_key *sk )
 }
 
 static int
-write_keybinding( KBNODE root, KBNODE pub_root, PKT_secret_key *sk )
+write_keybinding( KBNODE root, KBNODE pub_root, PKT_secret_key *sk,
+                  unsigned int usage )
 {
     PACKET *pkt;
     PKT_signature *sig;
     int rc=0;
     KBNODE node;
     PKT_public_key *pk, *subpk;
+    struct opaque_data_usage_and_pk oduap;
 
     if( opt.verbose )
 	log_info(_("writing key binding signature\n"));
@@ -218,6 +419,10 @@ write_keybinding( KBNODE root, KBNODE pub_root, PKT_secret_key *sk )
     if( !node )
 	BUG();
     pk = node->pkt->pkt.public_key;
+    /* we have to cache the key, so that the verification of the signature
+     * creation is able to retrieve the public key */
+    cache_public_key (pk);
+ 
     /* find the last subkey */
     subpk = NULL;
     for(node=pub_root; node; node = node->next ) {
@@ -228,8 +433,10 @@ write_keybinding( KBNODE root, KBNODE pub_root, PKT_secret_key *sk )
 	BUG();
 
     /* and make the signature */
-    rc = make_keysig_packet( &sig, pk, NULL, subpk, sk, 0x18, 0,
-				    keygen_add_key_expire, subpk );
+    oduap.usage = usage;
+    oduap.pk = subpk;
+    rc = make_keysig_packet( &sig, pk, NULL, subpk, sk, 0x18, 0, 0,
+        		     keygen_add_key_flags_and_expire, &oduap );
     if( rc ) {
 	log_error("make_keysig_packet failed: %s\n", g10_errstr(rc) );
 	return rc;
@@ -415,9 +622,10 @@ gen_dsa(unsigned int nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
     return 0;
 }
 
-#if 0
-/* we can't enable generation right now, becuase we first need to implement
- * the keyflags */
+
+/* 
+ * Generate an RSA key.
+ */
 static int
 gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 expireval )
@@ -426,7 +634,7 @@ gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
     PACKET *pkt;
     PKT_secret_key *sk;
     PKT_public_key *pk;
-    MPI skey[4];
+    MPI skey[6];
     MPI *factors;
 
     assert( is_RSA(algo) );
@@ -497,7 +705,6 @@ gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 
     return 0;
 }
-#endif
 
 
 /****************
@@ -530,13 +737,15 @@ check_valid_days( const char *s )
 
 /****************
  * Returns: 0 to create both a DSA and a ElGamal key.
+ *          and only if key flags are to be written the desired usage.
  */
 static int
-ask_algo( int addmode )
+ask_algo (int addmode, unsigned int *r_usage)
 {
     char *answer;
     int algo;
 
+    *r_usage = 0;
     tty_printf(_("Please select what kind of key you want:\n"));
     if( !addmode )
 	tty_printf(_("   (%d) DSA and ElGamal (default)\n"), 1 );
@@ -544,9 +753,9 @@ ask_algo( int addmode )
     if( addmode )
 	tty_printf(    _("   (%d) ElGamal (encrypt only)\n"), 3 );
     tty_printf(    _("   (%d) ElGamal (sign and encrypt)\n"), 4 );
-  #if 0
-    tty_printf(    _("   (%d) RSA (sign and encrypt)\n"), 5 );
-  #endif
+    tty_printf(    _("   (%d) RSA (sign only)\n"), 5 );
+    if (addmode)
+        tty_printf(    _("   (%d) RSA (encrypt only)\n"), 6 );
 
     for(;;) {
 	answer = cpr_get("keygen.algo",_("Your selection? "));
@@ -557,18 +766,19 @@ ask_algo( int addmode )
 	    algo = 0;	/* create both keys */
 	    break;
 	}
-      #if 0
+	else if( algo == 6 && addmode ) {
+	    algo = PUBKEY_ALGO_RSA;
+            *r_usage = PUBKEY_USAGE_ENC;
+	    break;
+	}
 	else if( algo == 5 ) {
-	    if( cpr_get_answer_is_yes("keygen.algo.rsa_se",_(
-		"Do you really want to create a sign and encrypt key? "))) {
-		algo = PUBKEY_ALGO_RSA;
-		break;
-	    }
+	    algo = PUBKEY_ALGO_RSA;
+            *r_usage = PUBKEY_USAGE_SIG;
+	    break;
 	}
-      #endif
 	else if( algo == 4 ) {
 	    if( cpr_get_answer_is_yes("keygen.algo.elg_se",_(
-		"Do you really want to create a sign and encrypt key? "))) {
+		"The use of this algorithm is deprecated - create anyway? "))){
 		algo = PUBKEY_ALGO_ELGAMAL;
 		break;
 	    }
@@ -594,11 +804,14 @@ ask_keysize( int algo )
     char *answer;
     unsigned nbits;
 
-    tty_printf(_("About to generate a new %s keypair.\n"
-		 "              minimum keysize is  768 bits\n"
-		 "              default keysize is 1024 bits\n"
-		 "    highest suggested keysize is 2048 bits\n"),
-					pubkey_algo_to_string(algo) );
+    if (algo != PUBKEY_ALGO_DSA) {
+        tty_printf (_("About to generate a new %s keypair.\n"
+                      "              minimum keysize is  768 bits\n"
+                      "              default keysize is 1024 bits\n"
+                      "    highest suggested keysize is 2048 bits\n"),
+                    pubkey_algo_to_string(algo) );
+    }
+
     for(;;) {
 	answer = cpr_get("keygen.size",
 			  _("What keysize do you want? (1024) "));
@@ -732,6 +945,8 @@ ask_expire_interval(void)
 	    /* print the date when the key expires */
 	    tty_printf(_("Key expires at %s\n"),
 			asctimestamp((ulong)(curtime + interval) ) );
+            /* FIXME: This check yields warning on alhas:
+               write a configure check and to this check here only for 32 bit machines */
 	    if( (time_t)((ulong)(curtime+interval)) < 0 )
 		tty_printf(_("Your system can't display dates beyond 2038.\n"
 		    "However, it will be correctly handled up to 2106.\n"));
@@ -876,7 +1091,7 @@ ask_user_id( int mode )
 	}
 
 	for(;;) {
-	    char *ansstr = _("NnCcEeOoQq");
+	    const char *ansstr = _("NnCcEeOoQq");
 
 	    if( strlen(ansstr) != 10 )
 		BUG();
@@ -972,7 +1187,7 @@ ask_passphrase( STRING2KEY **ret_s2k )
 
 
 static int
-do_create( int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root,
+do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
 	   DEK *dek, STRING2KEY *s2k, PKT_secret_key **sk, u32 expiredate )
 {
     int rc=0;
@@ -988,10 +1203,8 @@ do_create( int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root,
 	rc = gen_elg(algo, nbits, pub_root, sec_root, dek, s2k, sk, expiredate);
     else if( algo == PUBKEY_ALGO_DSA )
 	rc = gen_dsa(nbits, pub_root, sec_root, dek, s2k, sk, expiredate);
-  #if 0
     else if( algo == PUBKEY_ALGO_RSA )
 	rc = gen_rsa(algo, nbits, pub_root, sec_root, dek, s2k, sk, expiredate);
-  #endif
     else
 	BUG();
 
@@ -1074,6 +1287,38 @@ get_parameter_algo( struct para_data_s *para, enum para_name key )
     return string_to_pubkey_algo( r->u.value );
 }
 
+/* 
+ * parse the usage parameter and set the keyflags.  Return true on error.
+ */
+static int
+parse_parameter_usage (const char *fname,
+                       struct para_data_s *para, enum para_name key)
+{
+    struct para_data_s *r = get_parameter( para, key );
+    char *p, *pn;
+    unsigned int usage;
+
+    if( !r )
+	return 0; /* none (this is an optional parameter)*/
+    
+    usage = 0;
+    pn = r->u.value;
+    while ( (p = strsep (&pn, " \t,")) ) {
+        if ( !*p)
+            ;
+        else if ( !ascii_strcasecmp (p, "sign") )
+            usage |= PUBKEY_USAGE_SIG;
+        else if ( !ascii_strcasecmp (p, "encrypt") )
+            usage |= PUBKEY_USAGE_ENC;
+        else {
+            log_error("%s:%d: invalid usage list\n", fname, r->lnr );
+            return -1; /* error */
+        }
+    }
+    r->u.usage = usage;
+    return 0;
+}
+
 
 static u32
 get_parameter_u32( struct para_data_s *para, enum para_name key )
@@ -1084,6 +1329,8 @@ get_parameter_u32( struct para_data_s *para, enum para_name key )
 	return 0;
     if( r->key == pKEYEXPIRE || r->key == pSUBKEYEXPIRE )
 	return r->u.expire;
+    if( r->key == pKEYUSAGE || r->key == pSUBKEYUSAGE )
+	return r->u.usage;
 
     return (unsigned int)strtoul( r->u.value, NULL, 10 );
 }
@@ -1111,7 +1358,7 @@ get_parameter_s2k( struct para_data_s *para, enum para_name key )
 
 static int
 proc_parameter_file( struct para_data_s *para, const char *fname,
-				       struct output_control_s *outctrl )
+                     struct output_control_s *outctrl )
 {
     struct para_data_s *r;
     const char *s1, *s2, *s3;
@@ -1128,12 +1375,18 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
 	return -1;
     }
 
+    if (parse_parameter_usage (fname, para, pKEYUSAGE))
+        return -1;
+
     i = get_parameter_algo( para, pSUBKEYTYPE );
-    if( i > 1 && check_pubkey_algo( i ) ) {
+    if( i > 0 && check_pubkey_algo( i ) ) {
 	r = get_parameter( para, pSUBKEYTYPE );
 	log_error("%s:%d: invalid algorithm\n", fname, r->lnr );
 	return -1;
     }
+    if (i > 0 && parse_parameter_usage (fname, para, pSUBKEYUSAGE))
+        return -1;
+
 
     if( !get_parameter_value( para, pUSERID ) ) {
 	/* create the formatted user ID */
@@ -1227,8 +1480,10 @@ read_parameter_file( const char *fname )
     } keywords[] = {
 	{ "Key-Type",       pKEYTYPE},
 	{ "Key-Length",     pKEYLENGTH },
+	{ "Key-Usage",      pKEYUSAGE },
 	{ "Subkey-Type",    pSUBKEYTYPE },
 	{ "Subkey-Length",  pSUBKEYLENGTH },
+	{ "Subkey-Usage",   pSUBKEYUSAGE },
 	{ "Name-Real",      pNAMEREAL },
 	{ "Name-Email",     pNAMEEMAIL },
 	{ "Name-Comment",   pNAMECOMMENT },
@@ -1283,17 +1538,17 @@ read_parameter_file( const char *fname )
 		;
 	    value = p;
 	    trim_trailing_ws( value, strlen(value) );
-	    if( !stricmp( keyword, "%echo" ) )
+	    if( !ascii_strcasecmp( keyword, "%echo" ) )
 		log_info("%s\n", value );
-	    else if( !stricmp( keyword, "%dry-run" ) )
+	    else if( !ascii_strcasecmp( keyword, "%dry-run" ) )
 		outctrl.dryrun = 1;
-	    else if( !stricmp( keyword, "%commit" ) ) {
+	    else if( !ascii_strcasecmp( keyword, "%commit" ) ) {
 		outctrl.lnr = lnr;
 		proc_parameter_file( para, fname, &outctrl );
 		release_parameter_list( para );
 		para = NULL;
 	    }
-	    else if( !stricmp( keyword, "%pubring" ) ) {
+	    else if( !ascii_strcasecmp( keyword, "%pubring" ) ) {
 		if( outctrl.pub.fname && !strcmp( outctrl.pub.fname, value ) )
 		    ; /* still the same file - ignore it */
 		else {
@@ -1302,7 +1557,7 @@ read_parameter_file( const char *fname )
 		    outctrl.use_files = 1;
 		}
 	    }
-	    else if( !stricmp( keyword, "%secring" ) ) {
+	    else if( !ascii_strcasecmp( keyword, "%secring" ) ) {
 		if( outctrl.sec.fname && !strcmp( outctrl.sec.fname, value ) )
 		    ; /* still the same file - ignore it */
 		else {
@@ -1335,7 +1590,7 @@ read_parameter_file( const char *fname )
 	trim_trailing_ws( value, strlen(value) );
 
 	for(i=0; keywords[i].name; i++ ) {
-	    if( !stricmp( keywords[i].name, keyword ) )
+	    if( !ascii_strcasecmp( keywords[i].name, keyword ) )
 		break;
 	}
 	if( !keywords[i].name ) {
@@ -1407,6 +1662,7 @@ generate_keypair( const char *fname )
     DEK *dek;
     STRING2KEY *s2k;
     int algo;
+    unsigned int usage;
     int both = 0;
     u32 expire;
     struct para_data_s *para = NULL;
@@ -1420,7 +1676,7 @@ generate_keypair( const char *fname )
 	return;
     }
 
-    algo = ask_algo( 0 );
+    algo = ask_algo( 0, &usage );
     if( !algo ) { /* default: DSA with ElG subkey of the specified size */
 	both = 1;
 	r = m_alloc_clear( sizeof *r + 20 );
@@ -1448,6 +1704,17 @@ generate_keypair( const char *fname )
 	sprintf( r->u.value, "%d", algo );
 	r->next = para;
 	para = r;
+
+        if (usage) {
+            r = m_alloc_clear( sizeof *r + 20 );
+            r->key = pKEYUSAGE;
+            sprintf( r->u.value, "%s%s",
+                     (usage & PUBKEY_USAGE_SIG)? "sign ":"",
+                     (usage & PUBKEY_USAGE_ENC)? "encrypt ":"" );
+            r->next = para;
+            para = r;
+        }
+
     }
 
     nbits = ask_keysize( algo );
@@ -1504,13 +1771,12 @@ static void
 do_generate_keypair( struct para_data_s *para,
 		     struct output_control_s *outctrl )
 {
-    char *pub_fname = NULL;
-    char *sec_fname = NULL;
     KBNODE pub_root = NULL;
     KBNODE sec_root = NULL;
     PKT_secret_key *sk = NULL;
     const char *s;
     int rc;
+    int did_sub = 0;
 
     if( outctrl->dryrun ) {
 	log_info("dry-run mode - key generation skipped\n");
@@ -1557,20 +1823,14 @@ do_generate_keypair( struct para_data_s *para,
 						    &outctrl->sec.afx );
 	    }
 	}
-	pub_fname = outctrl->pub.fname; /* only for info output */
-	sec_fname = outctrl->sec.fname; /* only for info output */
 	assert( outctrl->pub.stream );
 	assert( outctrl->sec.stream );
-    }
-    else {
-	pub_fname = get_writable_keyblock_file( 0 );
-	sec_fname = get_writable_keyblock_file( 1 );
+        if( opt.verbose ) {
+            log_info(_("writing public key to `%s'\n"), outctrl->pub.fname );
+            log_info(_("writing secret key to `%s'\n"), outctrl->sec.fname );
+        }
     }
 
-    if( opt.verbose ) {
-	log_info(_("writing public key to `%s'\n"), pub_fname );
-	log_info(_("writing secret key to `%s'\n"), sec_fname );
-    }
 
     /* we create the packets as a tree of kbnodes. Because the structure
      * we create is known in advance we simply generate a linked list
@@ -1592,9 +1852,11 @@ do_generate_keypair( struct para_data_s *para,
 	if( !rc )
 	    write_uid(sec_root, s );
 	if( !rc )
-	    rc = write_selfsig(pub_root, pub_root, sk);
+	    rc = write_selfsig(pub_root, pub_root, sk,
+                               get_parameter_uint (para, pKEYUSAGE));
 	if( !rc )
-	    rc = write_selfsig(sec_root, pub_root, sk);
+	    rc = write_selfsig(sec_root, pub_root, sk,
+                               get_parameter_uint (para, pKEYUSAGE));
     }
 
     if( get_parameter( para, pSUBKEYTYPE ) ) {
@@ -1606,9 +1868,12 @@ do_generate_keypair( struct para_data_s *para,
 			NULL,
 			get_parameter_u32( para, pSUBKEYEXPIRE ) );
 	if( !rc )
-	    rc = write_keybinding(pub_root, pub_root, sk);
+	    rc = write_keybinding(pub_root, pub_root, sk,
+               			  get_parameter_uint (para, pSUBKEYUSAGE));
 	if( !rc )
-	    rc = write_keybinding(sec_root, pub_root, sk);
+	    rc = write_keybinding(sec_root, pub_root, sk,
+               			  get_parameter_uint (para, pSUBKEYUSAGE));
+        did_sub = 1;
     }
 
 
@@ -1624,50 +1889,57 @@ do_generate_keypair( struct para_data_s *para,
 
     }
     else if( !rc ) { /* write to the standard keyrings */
-	KBPOS pub_kbpos;
-	KBPOS sec_kbpos;
-	int rc1 = -1;
-	int rc2 = -1;
+	KEYDB_HANDLE pub_hd = keydb_new (0);
+	KEYDB_HANDLE sec_hd = keydb_new (1);
 
-	/* we can now write the certificates */
-	if( get_keyblock_handle( pub_fname, 0, &pub_kbpos ) ) {
-	    if( add_keyblock_resource( pub_fname, 1, 0 ) ) {
-		log_error("can add keyblock file `%s'\n", pub_fname );
-		rc = G10ERR_CREATE_FILE;
-	    }
-	    else if( get_keyblock_handle( pub_fname, 0, &pub_kbpos ) ) {
-		log_error("can get keyblock handle for `%s'\n", pub_fname );
-		rc = G10ERR_CREATE_FILE;
-	    }
-	}
-	if( rc )
-	    ;
-	else if( get_keyblock_handle( sec_fname, 1, &sec_kbpos ) ) {
-	    if( add_keyblock_resource( sec_fname, 1, 1 ) ) {
-		log_error("can add keyblock file `%s'\n", sec_fname );
-		rc = G10ERR_CREATE_FILE;
-	    }
-	    else if( get_keyblock_handle( sec_fname, 1, &sec_kbpos ) ) {
-		log_error("can get keyblock handle for `%s'\n", sec_fname );
-		rc = G10ERR_CREATE_FILE;
-	    }
-	}
+        /* FIXME: we may have to create the keyring first */
+        rc = keydb_locate_writable (pub_hd, NULL);
+        if (rc) 
+    	    log_error (_("no writable public keyring found: %s\n"),
+                       g10_errstr (rc));
+
+        if (!rc) {  
+            rc = keydb_locate_writable (sec_hd, NULL);
+            if (rc) 
+                log_error (_("no writable secret keyring found: %s\n"),
+                           g10_errstr (rc));
+        }
+
+        if (!rc && opt.verbose) {
+            log_info(_("writing public key to `%s'\n"),
+                     keydb_get_resource_name (pub_hd));
+            log_info(_("writing secret key to `%s'\n"),
+                     keydb_get_resource_name (sec_hd));
+        }
+
+        if (!rc) {
+	    rc = keydb_insert_keyblock (pub_hd, pub_root);
+            if (rc)
+                log_error (_("error writing public keyring `%s': %s\n"),
+                           keydb_get_resource_name (pub_hd), g10_errstr(rc));
+        }
+
+        if (!rc) {
+	    rc = keydb_insert_keyblock (sec_hd, sec_root);
+            if (rc)
+                log_error (_("error writing secret keyring `%s': %s\n"),
+                           keydb_get_resource_name (pub_hd), g10_errstr(rc));
+        }
+
+        keydb_release (pub_hd);
+        keydb_release (sec_hd);
+
+	if (!rc) {
+            int no_enc_rsa =
+                get_parameter_algo(para, pKEYTYPE) == PUBKEY_ALGO_RSA
+                && get_parameter_uint( para, pKEYUSAGE )
+                && !(get_parameter_uint( para,pKEYUSAGE) & PUBKEY_USAGE_ENC);
 
-	if( rc )
-	    ;
-	else if( (rc=rc1=lock_keyblock( &pub_kbpos )) )
-	    log_error("can't lock public keyring: %s\n", g10_errstr(rc) );
-	else if( (rc=rc2=lock_keyblock( &sec_kbpos )) )
-	    log_error("can't lock secret keyring: %s\n", g10_errstr(rc) );
-	else if( (rc=insert_keyblock( &pub_kbpos, pub_root )) )
-	    log_error("can't write public key: %s\n", g10_errstr(rc) );
-	else if( (rc=insert_keyblock( &sec_kbpos, sec_root )) )
-	    log_error("can't write secret key: %s\n", g10_errstr(rc) );
-	else {
 	    if( !opt.batch )
 		 tty_printf(_("public and secret key created and signed.\n") );
 	    if( !opt.batch
-		&& get_parameter_algo( para, pKEYTYPE ) == PUBKEY_ALGO_DSA
+		&& ( get_parameter_algo( para, pKEYTYPE ) == PUBKEY_ALGO_DSA
+                     || no_enc_rsa )
 		&& !get_parameter( para, pSUBKEYTYPE ) )
 	    {
 		tty_printf(_("Note that this key cannot be used for "
@@ -1676,11 +1948,6 @@ do_generate_keypair( struct para_data_s *para,
 			     "secondary key for this purpose.\n") );
 	    }
 	}
-
-	if( !rc1 )
-	    unlock_keyblock( &pub_kbpos );
-	if( !rc2 )
-	    unlock_keyblock( &sec_kbpos );
     }
 
     if( rc ) {
@@ -1689,14 +1956,13 @@ do_generate_keypair( struct para_data_s *para,
 	else
 	    tty_printf(_("Key generation failed: %s\n"), g10_errstr(rc) );
     }
+    else {
+        write_status_text (STATUS_KEY_CREATED, did_sub? "B":"P");
+    }
     release_kbnode( pub_root );
     release_kbnode( sec_root );
     if( sk ) /* the unprotected  secret key */
 	free_secret_key(sk);
-    if( !outctrl->use_files ) {
-	m_free(pub_fname);
-	m_free(sec_fname);
-    }
 }
 
 
@@ -1711,6 +1977,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     KBNODE node;
     PKT_secret_key *sk = NULL; /* this is the primary sk */
     int algo;
+    unsigned int usage;
     u32 expire;
     unsigned nbits;
     char *passphrase = NULL;
@@ -1741,6 +2008,9 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	}
     }
 
+    if (sk->version < 4) 
+        log_info (_("NOTE: creating subkeys for v3 keys "
+                    "is not OpenPGP compliant\n"));
 
     /* unprotect to get the passphrase */
     switch( is_secret_key_protected( sk ) ) {
@@ -1761,7 +2031,7 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	goto leave;
 
 
-    algo = ask_algo( 1 );
+    algo = ask_algo( 1, &usage );
     assert(algo);
     nbits = ask_keysize( algo );
     expire = ask_expire_interval();
@@ -1780,11 +2050,13 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
     rc = do_create( algo, nbits, pub_keyblock, sec_keyblock,
 				      dek, s2k, NULL, expire );
     if( !rc )
-	rc = write_keybinding(pub_keyblock, pub_keyblock, sk);
-    if( !rc )
-	rc = write_keybinding(sec_keyblock, pub_keyblock, sk);
+	rc = write_keybinding(pub_keyblock, pub_keyblock, sk, usage);
     if( !rc )
+	rc = write_keybinding(sec_keyblock, pub_keyblock, sk, usage);
+    if( !rc ) {
 	okay = 1;
+        write_status_text (STATUS_KEY_CREATED, "S");
+    }
 
   leave:
     if( rc )
