@@ -1,5 +1,5 @@
-/* keyid.c - jeyid and fingerprint handling
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+/* keyid.c - key ID and fingerprint handling
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -48,6 +48,17 @@ pubkey_letter( int algo )
     }
 }
 
+int
+get_lsign_letter ( PKT_signature *sig )
+{
+    const char *p;
+    
+    if (!sig)
+        return '?';
+    p = parse_sig_subpkt2( sig, SIGSUBPKT_EXPORTABLE, NULL );
+    return (p && !*p)? 'l':'x';
+}
+
 
 static MD_HANDLE
 do_fingerprint_md( PKT_public_key *pk )
@@ -260,6 +271,21 @@ nbits_from_sk( PKT_secret_key *sk )
     return pubkey_nbits( sk->pubkey_algo, sk->skey );
 }
 
+static const char *
+mk_datestr (char *buffer, time_t atime)
+{
+    struct tm *tp;
+
+    if ( atime < 0 ) /* 32 bit time_t and after 2038-01-19 */
+        strcpy (buffer, "????" "-??" "-??"); /* mark this as invalid */
+    else {
+        tp = gmtime (&atime);
+        sprintf (buffer,"%04d-%02d-%02d",
+                 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
+    }
+    return buffer;
+}
+
 /****************
  * return a string with the creation date of the pk
  * Note: this is alloced in a static buffer.
@@ -269,36 +295,27 @@ const char *
 datestr_from_pk( PKT_public_key *pk )
 {
     static char buffer[11+5];
-    struct tm *tp;
     time_t atime = pk->timestamp;
 
-    tp = gmtime( &atime );
-    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
-    return buffer;
+    return mk_datestr (buffer, atime);
 }
 
 const char *
 datestr_from_sk( PKT_secret_key *sk )
 {
     static char buffer[11+5];
-    struct tm *tp;
     time_t atime = sk->timestamp;
 
-    tp = gmtime( &atime );
-    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
-    return buffer;
+    return mk_datestr (buffer, atime);
 }
 
 const char *
 datestr_from_sig( PKT_signature *sig )
 {
     static char buffer[11+5];
-    struct tm *tp;
     time_t atime = sig->timestamp;
 
-    tp = gmtime( &atime );
-    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
-    return buffer;
+    return mk_datestr (buffer, atime);
 }
 
 
@@ -306,32 +323,73 @@ const char *
 expirestr_from_pk( PKT_public_key *pk )
 {
     static char buffer[11+5];
-    struct tm *tp;
     time_t atime;
 
     if( !pk->expiredate )
 	return _("never     ");
     atime = pk->expiredate;
-    tp = gmtime( &atime );
-    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
-    return buffer;
+    return mk_datestr (buffer, atime);
 }
 
 const char *
 expirestr_from_sk( PKT_secret_key *sk )
 {
     static char buffer[11+5];
-    struct tm *tp;
     time_t atime;
 
     if( !sk->expiredate )
 	return "never     ";
     atime = sk->expiredate;
-    tp = gmtime( &atime );
-    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
-    return buffer;
+    return mk_datestr (buffer, atime);
 }
 
+const char *
+colon_strtime (u32 t)
+{
+    if (!t)
+        return "";
+    if (opt.fixed_list_mode) {
+        static char buf[15];
+        sprintf (buf, "%lu", (ulong)t);
+        return buf;
+    }
+    return strtimestamp(t);
+}
+
+const char *
+colon_datestr_from_pk (PKT_public_key *pk)
+{
+    if (opt.fixed_list_mode) {
+        static char buf[15];
+        sprintf (buf, "%lu", (ulong)pk->timestamp);
+        return buf;
+    }
+    return datestr_from_pk (pk);
+}
+
+const char *
+colon_datestr_from_sk (PKT_secret_key *sk)
+{
+    if (opt.fixed_list_mode) {
+        static char buf[15];
+        sprintf (buf, "%lu", (ulong)sk->timestamp);
+        return buf;
+    }
+    return datestr_from_sk (sk);
+}
+
+const char *
+colon_datestr_from_sig (PKT_signature *sig)
+{
+    if (opt.fixed_list_mode) {
+        static char buf[15];
+        sprintf (buf, "%lu", (ulong)sig->timestamp);
+        return buf;
+    }
+    return datestr_from_sig (sig);
+}
+
+
 
 /**************** .
  * Return a byte array with the fingerprint for the given PK/SK
@@ -343,9 +401,9 @@ byte *
 fingerprint_from_pk( PKT_public_key *pk, byte *array, size_t *ret_len )
 {
     byte *p, *buf;
-    const char *dp;
+    const byte *dp;
     size_t len;
-    unsigned n;
+    unsigned int n;
 
     if( pk->version < 4 && is_RSA(pk->pubkey_algo) ) {
 	/* RSA in version 3 packets is special */
@@ -376,6 +434,8 @@ fingerprint_from_pk( PKT_public_key *pk, byte *array, size_t *ret_len )
 	if( !array )
 	    array = m_alloc( len );
 	memcpy(array, dp, len );
+	pk->keyid[0] = dp[12] << 24 | dp[13] << 16 | dp[14] << 8 | dp[15] ;
+	pk->keyid[1] = dp[16] << 24 | dp[17] << 16 | dp[18] << 8 | dp[19] ;
 	md_close(md);
     }
 
