@@ -1,5 +1,5 @@
 /* openfile.c
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -155,6 +155,8 @@ ask_outfile_name( const char *name, size_t namelen )
 	fname = defname; defname = NULL;
     }
     m_free(defname);
+    if (fname)
+        trim_spaces (fname);
     return fname;
 }
 
@@ -221,7 +223,19 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    name = buf;
 	}
 
-	if( overwrite_filep( name ) ) {
+        rc = 0;
+	while( !overwrite_filep (name) ) {
+            char *tmp = ask_outfile_name (NULL, 0);
+            if ( !tmp || !*tmp ) {
+                m_free (tmp);
+                rc = G10ERR_FILE_EXISTS;
+                break;
+            }
+            m_free (buf);
+            name = buf = tmp;
+        }
+
+	if( !rc ) {
 	    if( !(*a = iobuf_create( name )) ) {
 		log_error(_("%s: can't create: %s\n"), name, strerror(errno) );
 		rc = G10ERR_CREATE_FILE;
@@ -229,8 +243,6 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    else if( opt.verbose )
 		log_info(_("writing to `%s'\n"), name );
 	}
-	else
-	    rc = G10ERR_FILE_EXISTS;
 	m_free(buf);
     }
     return rc;
@@ -255,9 +267,9 @@ open_sigfile( const char *iname )
                         || !strcmp(iname + len - 4, ".asc")) ) {
 	    char *buf;
 	    buf = m_strdup(iname);
-	    buf[len-4] = 0 ;
+	    buf[len-(buf[len-1]=='n'?5:4)] = 0 ;
 	    a = iobuf_open( buf );
-	    if( opt.verbose )
+	    if( a && opt.verbose )
 		log_info(_("assuming signed data in `%s'\n"), buf );
 	    m_free(buf);
 	}
@@ -329,7 +341,7 @@ try_make_homedir( const char *fname )
 
     if ( ( *defhome == '~'
            && ( strlen(fname) >= strlen (defhome+1)
-                && !strcmp(fname+strlen(defhome+1)-strlen(defhome+1),
+                && !strcmp(fname+strlen(fname)-strlen(defhome+1),
                            defhome+1 ) ))
          || ( *defhome != '~'
               && !compare_filenames( fname, defhome ) )
@@ -345,7 +357,3 @@ try_make_homedir( const char *fname )
 	g10_exit(1);
     }
 }
-
-
-
-
