@@ -1,5 +1,5 @@
 /* options.h
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,7 +24,16 @@
 
 #undef ENABLE_COMMENT_PACKETS  /* don't create comment packets */
 
+#ifndef EXTERN_UNLESS_MAIN_MODULE
+/* Norcraft can't cope with common symbols */
+ #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
+  #define EXTERN_UNLESS_MAIN_MODULE extern
+ #else
+  #define EXTERN_UNLESS_MAIN_MODULE 
+ #endif
+#endif
 
+EXTERN_UNLESS_MAIN_MODULE
 struct {
     int verbose;
     int quiet;
@@ -41,10 +50,11 @@ struct {
     int check_sigs; /* check key signatures */
     int with_colons;
     int with_key_data;
+    int with_fingerprint; /* opt --with-fingerprint active */
     int fingerprint; /* list fingerprints */
     int list_sigs;   /* list signatures */
     int no_armor;
-    int list_packets; /* list-packets mode */
+    int list_packets; /* list-packets mode: 1=normal, 2=invoked by command*/
     int def_cipher_algo;
     int force_v3_sigs;
     int force_mdc;
@@ -89,7 +99,9 @@ struct {
     ulong set_filesize;
     int honor_http_proxy;
     int fast_list_mode;
+    int fixed_list_mode;
     int ignore_time_conflict;
+    int ignore_valid_from;
     int ignore_crc_error;
     int command_fd;
     int auto_key_retrieve;
@@ -98,6 +110,12 @@ struct {
     int use_agent;
     int merge_only;
     int try_all_secrets;
+    int no_expensive_trust_checks;
+    int no_sig_cache;
+    int no_sig_create_check;
+    int no_auto_check_trustdb;
+    int preserve_permissions;
+    int no_homedir_creation;
 } opt;
 
 
