@@ -1,5 +1,5 @@
 /* pkclist.c
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -31,6 +31,7 @@
 #include "keydb.h"
 #include "memory.h"
 #include "util.h"
+#include "main.h"
 #include "trustdb.h"
 #include "ttyio.h"
 #include "status.h"
@@ -39,62 +40,6 @@
 
 #define CONTROL_D ('D' - 'A' + 1)
 
-/* fixme: we have nearly the same code in keyedit.c */
-static void
-print_fpr( PKT_public_key *pk )
-{
-    byte array[MAX_FINGERPRINT_LEN], *p;
-    size_t i, n;
-
-    fingerprint_from_pk( pk, array, &n );
-    p = array;
-    /* Translators: this shoud fit into 24 bytes to that the fingerprint
-     * data is properly aligned with the user ID */
-    tty_printf(_("             Fingerprint:"));
-    if( n == 20 ) {
-	for(i=0; i < n ; i++, i++, p += 2 ) {
-	    if( i == 10 )
-		tty_printf(" ");
-	    tty_printf(" %02X%02X", *p, p[1] );
-	}
-    }
-    else {
-	for(i=0; i < n ; i++, p++ ) {
-	    if( i && !(i%8) )
-		tty_printf(" ");
-	    tty_printf(" %02X", *p );
-	}
-    }
-    tty_printf("\n");
-}
-
-static void
-fpr_info( PKT_public_key *pk )
-{
-    byte array[MAX_FINGERPRINT_LEN], *p;
-    size_t i, n;
-    FILE *fp = log_stream();
-
-    fingerprint_from_pk( pk, array, &n );
-    p = array;
-    log_info(_("Fingerprint:"));
-    if( n == 20 ) {
-	for(i=0; i < n ; i++, i++, p += 2 ) {
-	    if( i == 10 )
-		putc(' ', fp);
-	    fprintf(fp, " %02X%02X", *p, p[1] );
-	}
-    }
-    else {
-	for(i=0; i < n ; i++, p++ ) {
-	    if( i && !(i%8) )
-		putc(' ', fp);
-	    fprintf(fp, " %02X", *p );
-	}
-    }
-    putc('\n', fp );
-}
-
 
 /****************
  * Show the revocation reason as it is stored with the given signature
@@ -107,7 +52,7 @@ do_show_revocation_reason( PKT_signature *sig )
     int seq = 0;
     const char *text;
 
-    while( (p = enum_sig_subpkt( sig->hashed_data, SIGSUBPKT_REVOC_REASON,
+    while( (p = enum_sig_subpkt (sig->hashed, SIGSUBPKT_REVOC_REASON,
 				 &n, &seq )) ) {
 	if( !n )
 	    continue; /* invalid - just skip it */
@@ -204,8 +149,10 @@ show_revocation_reason( PKT_public_key *pk )
 
 
 static void
-show_paths( ulong lid, int only_first )
+show_paths (const PKT_public_key *pk, int only_first )
 {
+    log_debug("not yet implemented\n");
+#if 0    
     void *context = NULL;
     unsigned otrust, validity;
     int last_level, level;
@@ -223,6 +170,7 @@ show_paths( ulong lid, int only_first )
 	last_level = level;
 
 	rc = keyid_from_lid( lid, keyid );
+
 	if( rc ) {
 	    log_error("ooops: can't get keyid for lid %lu\n", lid);
 	    return;
@@ -261,6 +209,7 @@ show_paths( ulong lid, int only_first )
 	free_public_key( pk );
     }
     enum_cert_paths( &context, NULL, NULL, NULL ); /* release context */
+#endif
     tty_printf("\n");
 }
 
@@ -268,204 +217,182 @@ show_paths( ulong lid, int only_first )
 
 
 /****************
- * Returns true if an ownertrust has changed.
+ * mode: 0 = standard
+ *       1 = Without key info and additional menu option 'm'
+ *           this does also add an option to set the key to ultimately trusted.
+ * Returns: 
+ *      -2 = nothing changed - caller should show some additional info
+ *      -1 = quit operation
+ *       0 = nothing changed
+ *       1 = new ownertrust now in new_trust
  */
 static int
-do_edit_ownertrust( ulong lid, int mode, unsigned *new_trust, int defer_help )
+do_edit_ownertrust (PKT_public_key *pk, int mode,
+                    unsigned *new_trust, int defer_help )
 {
-    char *p;
-    int rc;
-    size_t n;
-    u32 keyid[2];
-    PKT_public_key *pk ;
-    int changed=0;
-    int quit=0;
-    int show=0;
-    int did_help=defer_help;
+  char *p;
+  size_t n;
+  u32 keyid[2];
+  int changed=0;
+  int quit=0;
+  int show=0;
+  int did_help=defer_help;
 
-    rc = keyid_from_lid( lid, keyid );
-    if( rc ) {
-	log_error("ooops: can't get keyid for lid %lu\n", lid);
-	return 0;
-    }
+  keyid_from_pk (pk, keyid);
+  for(;;) {
+    /* a string with valid answers */
+    const char *ans = _("sSmMqQ");
 
-    pk = m_alloc_clear( sizeof *pk );
-    rc = get_pubkey( pk, keyid );
-    if( rc ) {
-	log_error("key %08lX: public key not found: %s\n",
-				(ulong)keyid[1], g10_errstr(rc) );
-	return 0;
-    }
-
-
-    for(;;) {
-	/* a string with valid answers */
-	const char *ans = _("sSmMqQ");
-
-	if( !did_help ) {
-	    if( !mode ) {
-		tty_printf(_("No trust value assigned to %lu:\n"
-			   "%4u%c/%08lX %s \""), lid,
-			  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
-			  (ulong)keyid[1], datestr_from_pk( pk ) );
-		p = get_user_id( keyid, &n );
-		tty_print_utf8_string( p, n ),
-		m_free(p);
-		tty_printf("\"\n");
-		print_fpr( pk );
-		tty_printf("\n");
-	    }
-	    tty_printf(_(
-"Please decide how far you trust this user to correctly\n"
-"verify other users' keys (by looking at passports,\n"
-"checking fingerprints from different sources...)?\n\n"
-" 1 = Don't know\n"
-" 2 = I do NOT trust\n"
-" 3 = I trust marginally\n"
-" 4 = I trust fully\n"
-" s = please show me more information\n") );
-	    if( mode )
-		tty_printf(_(" m = back to the main menu\n"));
-	    else
-		tty_printf(_(" q = quit\n"));
-	    tty_printf("\n");
-	    did_help = 1;
-	}
-	if( strlen(ans) != 6 )
-	    BUG();
-	p = cpr_get("edit_ownertrust.value",_("Your decision? "));
-	trim_spaces(p);
-	cpr_kill_prompt();
-	if( !*p )
-	    did_help = 0;
-	else if( *p && p[1] )
-	    ;
-	else if( !p[1] && (*p >= '1' && *p <= '4') ) {
-	    unsigned trust;
-	    switch( *p ) {
-	      case '1': trust = TRUST_UNDEFINED; break;
-	      case '2': trust = TRUST_NEVER    ; break;
-	      case '3': trust = TRUST_MARGINAL ; break;
-	      case '4': trust = TRUST_FULLY    ; break;
-	      default: BUG();
-	    }
+    if( !did_help ) 
+      {
+        if( !mode ) 
+          {
+            tty_printf(_("No trust value assigned to:\n"
+                         "%4u%c/%08lX %s \""),
+                       nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
+                       (ulong)keyid[1], datestr_from_pk( pk ) );
+            p = get_user_id( keyid, &n );
+            tty_print_utf8_string( p, n ),
+              m_free(p);
+            tty_printf("\"\n");
+            print_fingerprint (pk, NULL, 2);
+            tty_printf("\n");
+          }
+        tty_printf (_(
+                     "Please decide how far you trust this user to correctly\n"
+                     "verify other users' keys (by looking at passports,\n"
+                     "checking fingerprints from different sources...)?\n\n"));
+        tty_printf (_(" %d = Don't know\n"), 1);
+        tty_printf (_(" %d = I do NOT trust\n"), 2);
+        tty_printf (_(" %d = I trust marginally\n"), 3);
+        tty_printf (_(" %d = I trust fully\n"), 4);
+        if (mode)
+          tty_printf (_(" %d = I trust ultimately\n"), 5);
+        tty_printf (_(" s = please show me more information\n") );
+        if( mode )
+          tty_printf(_(" m = back to the main menu\n"));
+        else
+          tty_printf(_(" q = quit\n"));
+        tty_printf("\n");
+        did_help = 1;
+      }
+    if( strlen(ans) != 6 )
+      BUG();
+    p = cpr_get("edit_ownertrust.value",_("Your decision? "));
+    trim_spaces(p);
+    cpr_kill_prompt();
+    if( !*p )
+      did_help = 0;
+    else if( *p && p[1] )
+      ;
+    else if( !p[1] && (*p >= '1' && *p <= (mode?'5':'4')) ) 
+      {
+        unsigned int trust;
+        switch( *p )
+          {
+          case '1': trust = TRUST_UNDEFINED; break;
+          case '2': trust = TRUST_NEVER    ; break;
+          case '3': trust = TRUST_MARGINAL ; break;
+          case '4': trust = TRUST_FULLY    ; break;
+          case '5': trust = TRUST_ULTIMATE ; break;
+          default: BUG();
+          }
+        if (trust == TRUST_ULTIMATE
+            && !cpr_get_answer_is_yes ("edit_ownertrust.set_ultimate.okay",
+                                       _("Do you really want to set this key"
+                                         " to ultimate trust? ")))
+          ; /* no */
+        else
+          {
             *new_trust = trust;
             changed = 1;
             break;
-	}
-	else if( *p == ans[0] || *p == ans[1] ) {
-	    tty_printf(_(
-		"Certificates leading to an ultimately trusted key:\n"));
-	    show = 1;
-	    break;
-	}
-	else if( mode && (*p == ans[2] || *p == ans[3] || *p == CONTROL_D ) ) {
-	    break ; /* back to the menu */
-	}
-	else if( !mode && (*p == ans[4] || *p == ans[5] ) ) {
-	    quit = 1;
-	    break ; /* back to the menu */
-	}
-	m_free(p); p = NULL;
-    }
-    m_free(p);
-    m_free(pk);
-    return show? -2: quit? -1 : changed;
+          }
+      }
+    else if( *p == ans[0] || *p == ans[1] ) 
+      {
+        tty_printf(_("Certificates leading to an ultimately trusted key:\n"));
+        show = 1;
+        break;
+      }
+    else if( mode && (*p == ans[2] || *p == ans[3] || *p == CONTROL_D ) ) 
+      {
+        break ; /* back to the menu */
+      }
+    else if( !mode && (*p == ans[4] || *p == ans[5] ) )
+      {
+        quit = 1;
+        break ; /* back to the menu */
+      }
+    m_free(p); p = NULL;
+  }
+  m_free(p);
+  return show? -2: quit? -1 : changed;
 }
 
-
-int
-edit_ownertrust( ulong lid, int mode )
-{
-    unsigned int trust;
-    int no_help = 0;
-
-    for(;;) {
-	switch( do_edit_ownertrust( lid, mode, &trust, no_help ) ) {
-	  case -1:
-	    return 0;
-	  case -2:
-	    show_paths( lid, 1	);
-	    no_help = 1;
-	    break;
-	  case 1:
-	    trust &= ~TRUST_FLAG_DISABLED;
-	    trust |= get_ownertrust( lid ) & TRUST_FLAG_DISABLED;
-	    if( !update_ownertrust( lid, trust ) )
-		return 1;
-	    return 0;
-	  default:
-	    return 0;
-	}
-    }
-}
-
-static int
-add_ownertrust_cb( ulong lid )
-{
-    unsigned trust;
-    int rc = do_edit_ownertrust( lid, 0, &trust, 0 );
-
-    if( rc == 1 )
-	return trust & TRUST_MASK;
-    return rc > 0? 0 : rc;
-}
-
-/****************
- * Try to add some more owner trusts (interactive)
- * This function presents all the signator in a certificate
- * chain who have no ownertrust value assigned.
- * Returns: -1 if no ownertrust were added.
+/* 
+ * Display a menu to change the ownertrust of the key PK (which should
+ * be a primary key).  
+ * For mode values see do_edit_ownertrust ()
  */
-static int
-add_ownertrust( PKT_public_key *pk, int *quit, unsigned *trustlevel )
+int
+edit_ownertrust (PKT_public_key *pk, int mode )
 {
-    int rc;
-    unsigned flags = 0;
+  unsigned int trust;
+  int no_help = 0;
 
-    *quit = 0;
-    *trustlevel = 0;
-    tty_printf(
-_("Could not find a valid trust path to the key.  Let's see whether we\n"
-  "can assign some missing owner trust values.\n\n"));
-
-    rc = check_trust( pk, trustlevel, NULL, add_ownertrust_cb, &flags );
-
-    if( !(flags & 1) )
-	tty_printf(_("No path leading to one of our keys found.\n\n") );
-    else if( !(flags & 2) )
-	tty_printf(_("No certificates with undefined trust found.\n\n") );
-    else if( !(flags & 4) )
-	tty_printf(_("No trust values changed.\n\n") );
-
-    return (flags & 4)? 0:-1;
+  for(;;)
+    {
+      switch ( do_edit_ownertrust (pk, mode, &trust, no_help ) )
+        {
+        case -1: /* quit */
+          return 0;
+        case -2: /* show info */
+          show_paths(pk, 1);
+          no_help = 1;
+          break;
+        case 1: /* trust value set */
+          trust &= ~TRUST_FLAG_DISABLED;
+          trust |= get_ownertrust (pk) & TRUST_FLAG_DISABLED;
+          update_ownertrust (pk, trust );
+          return 0;
+        default:
+          return 0;
+        }
+    }
 }
 
+
 /****************
  * Check whether we can trust this pk which has a trustlevel of TRUSTLEVEL
- * Returns: true if we trust. Might change the trustlevel
+ * Returns: true if we trust.
  */
 static int
-do_we_trust( PKT_public_key *pk, int *trustlevel )
+do_we_trust( PKT_public_key *pk, unsigned int *trustlevel )
 {
-    int rc;
-    int did_add = 0;
-    int trustmask = 0;
+    unsigned int trustmask = 0;
 
-  retry:
+    /* FIXME: get_pubkey_byname already checks the validity and won't
+     * return keys which are either expired or revoked - so these
+     * question here won't get triggered.  We have to find a solution
+     * for this.  It might make sense to have a function in getkey.c
+     * which does only the basic checks and returns even revoked and
+     * expired keys.  This fnction could then also returhn a list of
+     * keys if the speicified name is ambiguous
+     */
     if( (*trustlevel & TRUST_FLAG_REVOKED) ) {
 	log_info(_("key %08lX: key has been revoked!\n"),
 					(ulong)keyid_from_pk( pk, NULL) );
 	show_revocation_reason( pk );
 	if( opt.batch )
-	    return 0;
+          return 0; /* no */
 
 	if( !cpr_get_answer_is_yes("revoked_key.override",
 				    _("Use this key anyway? ")) )
-	    return 0;
+          return 0; /* no */
 	trustmask |= TRUST_FLAG_REVOKED;
     }
-    else if( (*trustlevel & TRUST_FLAG_SUB_REVOKED) ) {
+    if( (*trustlevel & TRUST_FLAG_SUB_REVOKED) ) {
 	log_info(_("key %08lX: subkey has been revoked!\n"),
 					(ulong)keyid_from_pk( pk, NULL) );
 	show_revocation_reason( pk );
@@ -482,52 +409,25 @@ do_we_trust( PKT_public_key *pk, int *trustlevel )
     if( opt.always_trust) {
 	if( opt.verbose )
 	    log_info("No trust check due to --always-trust option\n");
-	/* The problem with this, is that EXPIRE can't be checked as
-	 * this needs to insert a ne key into the trustdb first and
-	 * we don't want that */
 	return 1;
     }
 
-
     switch( (*trustlevel & TRUST_MASK) ) {
-      case TRUST_UNKNOWN: /* No pubkey in trustDB: Insert and check again */
-	rc = insert_trust_record_by_pk( pk );
-	if( rc ) {
-	    log_error("failed to insert it into the trustdb: %s\n",
-						      g10_errstr(rc) );
-	    return 0; /* no */
-	}
-	rc = check_trust( pk, trustlevel, NULL, NULL, NULL );
-	*trustlevel &= ~trustmask;
-	if( rc )
-	    log_fatal("trust check after insert failed: %s\n",
-						      g10_errstr(rc) );
-	if( *trustlevel == TRUST_UNKNOWN || *trustlevel == TRUST_EXPIRED ) {
-	    log_debug("do_we_trust: oops at %d\n", __LINE__ );
-	    return 0;
-	}
-	return do_we_trust( pk, trustlevel );
-
       case TRUST_EXPIRED:
 	log_info(_("%08lX: key has expired\n"),
 				    (ulong)keyid_from_pk( pk, NULL) );
 	return 0; /* no */
 
+      default:
+         log_error ("invalid trustlevel %u returned from validation layer\n",
+                    *trustlevel);
+         /* fall thru */
+      case TRUST_UNKNOWN: 
       case TRUST_UNDEFINED:
-	if( opt.batch || opt.answer_no )
-	    log_info(_("%08lX: no info to calculate a trust probability\n"),
-					(ulong)keyid_from_pk( pk, NULL) );
-	else {
-	    int quit;
-
-	    rc = add_ownertrust( pk, &quit, trustlevel );
-	    *trustlevel &= ~trustmask;
-	    if( !rc && !did_add && !quit ) {
-		did_add = 1;
-		goto retry;
-	    }
-	}
-	return 0;
+        log_info(_("%08lX: There is no indication that this key "
+                   "really belongs to the owner\n"),
+                 (ulong)keyid_from_pk( pk, NULL) );
+	return 0; /* no */
 
       case TRUST_NEVER:
 	log_info(_("%08lX: We do NOT trust this key\n"),
@@ -549,8 +449,6 @@ do_we_trust( PKT_public_key *pk, int *trustlevel )
 	if( opt.verbose )
 	    log_info(_("This key belongs to us\n"));
 	return 1; /* yes */
-
-      default: BUG();
     }
 
     return 1; /* yes */
@@ -563,7 +461,7 @@ do_we_trust( PKT_public_key *pk, int *trustlevel )
  * key anyway.
  */
 static int
-do_we_trust_pre( PKT_public_key *pk, int trustlevel )
+do_we_trust_pre( PKT_public_key *pk, unsigned int trustlevel )
 {
     int rc;
 
@@ -573,7 +471,8 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 	return 0;
     if( (trustlevel & TRUST_FLAG_SUB_REVOKED) && !rc )
 	return 0;
-    else if( !opt.batch && !rc ) {
+
+    if( !opt.batch && !rc ) {
 	char *p;
 	u32 keyid[2];
 	size_t n;
@@ -586,7 +485,7 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 	tty_print_utf8_string( p, n ),
 	m_free(p);
 	tty_printf("\"\n");
-	print_fpr( pk );
+        print_fingerprint (pk, NULL, 2);
 	tty_printf("\n");
 
 	tty_printf(_(
@@ -598,7 +497,7 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 				  _("Use this key anyway? "))  )
 	    rc = 1;
 
-	/* Hmmm: Should we set a flag to tell the user the user about
+	/* Hmmm: Should we set a flag to tell the user about
 	 *	 his decision the next time he encrypts for this recipient?
 	 */
     }
@@ -619,121 +518,105 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 int
 check_signatures_trust( PKT_signature *sig )
 {
-    PKT_public_key *pk = m_alloc_clear( sizeof *pk );
-    int trustlevel;
-    int did_add = 0;
-    int rc=0;
+  PKT_public_key *pk = m_alloc_clear( sizeof *pk );
+  unsigned int trustlevel;
+  int did_add = 0;
+  int rc=0;
 
-
-    if( opt.always_trust ) {
-	if( !opt.quiet )
-	    log_info(_("WARNING: Using untrusted key!\n"));
-	return 0;
+  if ( opt.always_trust)
+    {
+      if( !opt.quiet )
+        log_info(_("WARNING: Using untrusted key!\n"));
+      if (opt.with_fingerprint)
+        print_fingerprint (pk, NULL, 1);
+      goto leave;
     }
 
-
-    rc = get_pubkey( pk, sig->keyid );
-    if( rc ) { /* this should not happen */
-	log_error("Ooops; the key vanished  - can't check the trust\n");
-	rc = G10ERR_NO_PUBKEY;
-	goto leave;
+  rc = get_pubkey( pk, sig->keyid );
+  if (rc) 
+    { /* this should not happen */
+      log_error("Ooops; the key vanished  - can't check the trust\n");
+      rc = G10ERR_NO_PUBKEY;
+      goto leave;
     }
 
-    rc = check_trust( pk, &trustlevel, NULL, NULL, NULL );
-    if( rc ) {
-	log_error("check trust failed: %s\n", g10_errstr(rc));
-	goto leave;
+  trustlevel = get_validity (pk, NULL);
+
+  if ( (trustlevel & TRUST_FLAG_REVOKED) ) 
+    {
+      write_status( STATUS_KEYREVOKED );
+      log_info(_("WARNING: This key has been revoked by its owner!\n"));
+      log_info(_("         This could mean that the signature is forgery.\n"));
+      show_revocation_reason( pk );
+    }
+  else if ((trustlevel & TRUST_FLAG_SUB_REVOKED) ) 
+    {
+      write_status( STATUS_KEYREVOKED );
+      log_info(_("WARNING: This subkey has been revoked by its owner!\n"));
+      show_revocation_reason( pk );
+    }
+  
+  if ((trustlevel & TRUST_FLAG_DISABLED))
+    log_info (_("Note: This key has been disabled.\n"));
+
+  switch ( (trustlevel & TRUST_MASK) ) 
+    {
+    case TRUST_EXPIRED:
+      log_info(_("Note: This key has expired!\n"));
+      print_fingerprint (pk, NULL, 1);
+      break;
+        
+    default:
+      log_error ("invalid trustlevel %u returned from validation layer\n",
+                 trustlevel);
+      /* fall thru */
+    case TRUST_UNKNOWN: 
+    case TRUST_UNDEFINED:
+      if( did_add || opt.batch || opt.answer_no ) {
+        write_status( STATUS_TRUST_UNDEFINED );
+        log_info(_("WARNING: This key is not certified with"
+                   " a trusted signature!\n"));
+        log_info(_("         There is no indication that the "
+                   "signature belongs to the owner.\n" ));
+        print_fingerprint (pk, NULL, 1);
+      }
+      break;
+
+    case TRUST_NEVER:
+      /* currently we won't get that status */
+      write_status( STATUS_TRUST_NEVER );
+      log_info(_("WARNING: We do NOT trust this key!\n"));
+      log_info(_("         The signature is probably a FORGERY.\n"));
+      if (opt.with_fingerprint)
+        print_fingerprint (pk, NULL, 1);
+      rc = G10ERR_BAD_SIGN;
+      break;
+
+    case TRUST_MARGINAL:
+      write_status( STATUS_TRUST_MARGINAL );
+      log_info(_("WARNING: This key is not certified with"
+                 " sufficiently trusted signatures!\n"));
+      log_info(_("         It is not certain that the"
+                 " signature belongs to the owner.\n" ));
+      print_fingerprint (pk, NULL, 1);
+      break;
+
+    case TRUST_FULLY:
+      write_status( STATUS_TRUST_FULLY );
+      if (opt.with_fingerprint)
+        print_fingerprint (pk, NULL, 1);
+      break;
+
+    case TRUST_ULTIMATE:
+      write_status( STATUS_TRUST_ULTIMATE );
+      if (opt.with_fingerprint)
+        print_fingerprint (pk, NULL, 1);
+      break;
     }
 
-  retry:
-    if( (trustlevel & TRUST_FLAG_REVOKED) ) {
-	write_status( STATUS_KEYREVOKED );
-	log_info(_("WARNING: This key has been revoked by its owner!\n"));
-	log_info(_("         This could mean that the signature is forgery.\n"));
-	show_revocation_reason( pk );
-    }
-    else if( (trustlevel & TRUST_FLAG_SUB_REVOKED) ) {
-	write_status( STATUS_KEYREVOKED );
-	log_info(_("WARNING: This subkey has been revoked by its owner!\n"));
-	show_revocation_reason( pk );
-    }
-
-
-    switch( (trustlevel & TRUST_MASK) ) {
-      case TRUST_UNKNOWN: /* No pubkey in trustDB: Insert and check again */
-	rc = insert_trust_record_by_pk( pk );
-	if( rc ) {
-	    log_error("failed to insert it into the trustdb: %s\n",
-						      g10_errstr(rc) );
-	    goto leave;
-	}
-	rc = check_trust( pk, &trustlevel, NULL, NULL, NULL );
-	if( rc )
-	    log_fatal("trust check after insert failed: %s\n",
-						      g10_errstr(rc) );
-	if( trustlevel == TRUST_UNKNOWN || trustlevel == TRUST_EXPIRED )
-	    BUG();
-	goto retry;
-
-      case TRUST_EXPIRED:
-	log_info(_("Note: This key has expired!\n"));
-	fpr_info( pk );
-	break;
-
-      case TRUST_UNDEFINED:
-	if( did_add || opt.batch || opt.answer_no ) {
-	    write_status( STATUS_TRUST_UNDEFINED );
-	    log_info(_(
-	    "WARNING: This key is not certified with a trusted signature!\n"));
-	    log_info(_(
-	    "         There is no indication that the "
-				    "signature belongs to the owner.\n" ));
-	    fpr_info( pk );
-	}
-	else {
-	    int quit;
-	    rc = add_ownertrust( pk, &quit, &trustlevel );
-	    if( rc || quit ) {
-		did_add = 1;
-		rc = 0;
-	    }
-	    goto retry;
-	}
-	break;
-
-      case TRUST_NEVER:
-	write_status( STATUS_TRUST_NEVER );
-	log_info(_("WARNING: We do NOT trust this key!\n"));
-	log_info(_("         The signature is probably a FORGERY.\n"));
-	rc = G10ERR_BAD_SIGN;
-	break;
-
-      case TRUST_MARGINAL:
-	write_status( STATUS_TRUST_MARGINAL );
-	log_info(_(
-	 "WARNING: This key is not certified with sufficiently trusted signatures!\n"
-		));
-	log_info(_(
-	 "         It is not certain that the signature belongs to the owner.\n"
-		 ));
-	fpr_info( pk );
-	break;
-
-      case TRUST_FULLY:
-	write_status( STATUS_TRUST_FULLY );
-	break;
-
-      case TRUST_ULTIMATE:
-	write_status( STATUS_TRUST_ULTIMATE );
-	break;
-
-      default: BUG();
-    }
-
-
-  leave:
-    free_public_key( pk );
-    return rc;
+ leave:
+  free_public_key( pk );
+  return rc;
 }
 
 
@@ -813,13 +696,14 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	    any_recipients = 1;
 	else if( (use & PUBKEY_USAGE_ENC) && !opt.no_encrypt_to ) {
 	    pk = m_alloc_clear( sizeof *pk );
-	    pk->pubkey_usage = use;
-	    if( (rc = get_pubkey_byname( NULL, pk, rov->d, NULL )) ) {
+	    pk->req_usage = use;
+	    if( (rc = get_pubkey_byname( pk, rov->d, NULL, NULL )) ) {
 		free_public_key( pk ); pk = NULL;
 		log_error(_("%s: skipped: %s\n"), rov->d, g10_errstr(rc) );
-	    }
+                write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
+                                              rov->d, strlen (rov->d), -1);
+            }
 	    else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use )) ) {
-
 		/* Skip the actual key if the key is already present
 		 * in the list */
 		if (key_present_in_pk_list(pk_list, pk) == 0) {
@@ -839,6 +723,8 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	    else {
 		free_public_key( pk ); pk = NULL;
 		log_error(_("%s: skipped: %s\n"), rov->d, g10_errstr(rc) );
+                write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
+                                              rov->d, strlen (rov->d), -1);
 	    }
 	}
     }
@@ -870,8 +756,8 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	    if( pk )
 		free_public_key( pk );
 	    pk = m_alloc_clear( sizeof *pk );
-	    pk->pubkey_usage = use;
-	    rc = get_pubkey_byname( NULL, pk, answer, NULL );
+	    pk->req_usage = use;
+	    rc = get_pubkey_byname( pk, answer, NULL, NULL );
 	    if( rc )
 		tty_printf(_("No such user ID.\n"));
 	    else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) ) {
@@ -894,13 +780,8 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 		else {
 		    int trustlevel;
 
-		    rc = check_trust( pk, &trustlevel, pk->namehash,
-						       NULL, NULL );
-		    if( rc ) {
-			log_error("error checking pk of `%s': %s\n",
-						     answer, g10_errstr(rc) );
-		    }
-		    else if( (trustlevel & TRUST_FLAG_DISABLED) ) {
+		    trustlevel = get_validity (pk, NULL);
+		    if( (trustlevel & TRUST_FLAG_DISABLED) ) {
 			tty_printf(_("Public key is disabled.\n") );
 		    }
 		    else if( do_we_trust_pre( pk, trustlevel ) ) {
@@ -936,8 +817,8 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
     }
     else if( !any_recipients && (def_rec = default_recipient()) ) {
 	pk = m_alloc_clear( sizeof *pk );
-	pk->pubkey_usage = use;
-	rc = get_pubkey_byname( NULL, pk, def_rec, NULL );
+	pk->req_usage = use;
+	rc = get_pubkey_byname( pk, def_rec, NULL, NULL );
 	if( rc )
 	    log_error(_("unknown default recipient `%s'\n"), def_rec );
 	else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) ) {
@@ -961,24 +842,26 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 		continue; /* encrypt-to keys are already handled */
 
 	    pk = m_alloc_clear( sizeof *pk );
-	    pk->pubkey_usage = use;
-	    if( (rc = get_pubkey_byname( NULL, pk, remusr->d, NULL )) ) {
+	    pk->req_usage = use;
+	    if( (rc = get_pubkey_byname( pk, remusr->d, NULL, NULL )) ) {
 		free_public_key( pk ); pk = NULL;
 		log_error(_("%s: skipped: %s\n"), remusr->d, g10_errstr(rc) );
+                write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
+                                              remusr->d, strlen (remusr->d),
+                                              -1);
 	    }
 	    else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use )) ) {
 		int trustlevel;
 
-		rc = check_trust( pk, &trustlevel, pk->namehash, NULL, NULL );
-		if( rc ) {
-		    free_public_key( pk ); pk = NULL;
-		    log_error(_("%s: error checking key: %s\n"),
-						      remusr->d, g10_errstr(rc) );
-		}
-		else if( (trustlevel & TRUST_FLAG_DISABLED) ) {
+		trustlevel = get_validity (pk, pk->namehash);
+		if( (trustlevel & TRUST_FLAG_DISABLED) ) {
 		    free_public_key(pk); pk = NULL;
 		    log_info(_("%s: skipped: public key is disabled\n"),
 								    remusr->d);
+                    write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
+                                                  remusr->d,
+                                                  strlen (remusr->d),
+                                                  -1);
 		}
 		else if( do_we_trust_pre( pk, trustlevel ) ) {
 		    /* note: do_we_trust may have changed the trustlevel */
@@ -1005,10 +888,18 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 		}
 		else { /* we don't trust this pk */
 		    free_public_key( pk ); pk = NULL;
+                    write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
+                                                  remusr->d,
+                                                  strlen (remusr->d),
+                                                  -1);
 		}
 	    }
 	    else {
 		free_public_key( pk ); pk = NULL;
+                write_status_text_and_buffer (STATUS_INV_RECP, "0 ",
+                                              remusr->d,
+                                              strlen (remusr->d),
+                                              -1);
 		log_error(_("%s: skipped: %s\n"), remusr->d, g10_errstr(rc) );
 	    }
 	}
@@ -1016,6 +907,7 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 
     if( !rc && !any_recipients ) {
 	log_error(_("no valid addressees\n"));
+        write_status_text (STATUS_NO_RECP, "0");
 	rc = G10ERR_NO_USER_ID;
     }
 
@@ -1037,13 +929,15 @@ algo_available( int preftype, int algo )
     else if( preftype == PREFTYPE_HASH ) {
 	return algo && !check_digest_algo( algo );
     }
-    else if( preftype == PREFTYPE_COMPR ) {
+    else if( preftype == PREFTYPE_ZIP ) {
 	return !algo || algo == 1 || algo == 2;
     }
     else
 	return 0;
 }
 
+
+
 /****************
  * Return -1 if we could not find an algorithm.
  */
@@ -1052,8 +946,7 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 {
     PK_LIST pkr;
     u32 bits[8];
-    byte *pref = NULL;
-    size_t npref;
+    const prefitem_t *prefs;
     int i, j;
     int compr_hack=0;
     int any;
@@ -1066,43 +959,38 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	u32 mask[8];
 
 	memset( mask, 0, 8 * sizeof *mask );
-	if( !pkr->pk->local_id ) { /* try to set the local id */
-	    query_trust_info( pkr->pk, NULL );
-	    if( !pkr->pk->local_id ) {
-		log_debug("select_algo_from_prefs: can't get LID\n");
-		continue;
-	    }
-	}
 	if( preftype == PREFTYPE_SYM )
 	    mask[0] |= (1<<2); /* 3DES is implicitly there */
-	m_free(pref);
-	pref = get_pref_data( pkr->pk->local_id, pkr->pk->namehash, &npref);
+        
+        if (pkr->pk->user_id) /* selected by user ID */
+            prefs = pkr->pk->user_id->prefs;
+        else
+            prefs = pkr->pk->prefs;
+
 	any = 0;
-	if( pref ) {
-	   #if 0
-	    log_hexdump("raw: ", pref, npref );
-	   #endif
-	    for(i=0; i+1 < npref; i+=2 ) {
-		if( pref[i] == preftype ) {
-		    mask[pref[i+1]/32] |= 1 << (pref[i+1]%32);
+	if( prefs ) {
+	    for (i=0; prefs[i].type; i++ ) {
+		if( prefs[i].type == preftype ) {
+		    mask[prefs[i].value/32] |= 1 << (prefs[i].value%32);
 		    any = 1;
 		}
 	    }
 	}
-	if( (!pref || !any) && preftype == PREFTYPE_COMPR ) {
+
+	if( (!prefs || !any) && preftype == PREFTYPE_ZIP ) {
 	    mask[0] |= 3; /* asume no_compression and old pgp */
 	    compr_hack = 1;
 	}
 
       #if 0
-	log_debug("mask=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
+	log_debug("pref mask=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
 	       (ulong)mask[7], (ulong)mask[6], (ulong)mask[5], (ulong)mask[4],
 	     (ulong)mask[3], (ulong)mask[2], (ulong)mask[1], (ulong)mask[0]);
       #endif
 	for(i=0; i < 8; i++ )
 	    bits[i] &= mask[i];
       #if 0
-	log_debug("bits=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
+	log_debug("pref bits=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
 	       (ulong)bits[7], (ulong)bits[6], (ulong)bits[5], (ulong)bits[4],
 	     (ulong)bits[3], (ulong)bits[2], (ulong)bits[1], (ulong)bits[0]);
       #endif
@@ -1115,20 +1003,20 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
      */
     i = -1;
     any = 0;
-    if( pref ) {
-	for(j=0; j+1 < npref; j+=2 ) {
-	    if( pref[j] == preftype ) {
-		if( (bits[pref[j+1]/32] & (1<<(pref[j+1]%32))) ) {
-		    if( algo_available( preftype, pref[j+1] ) ) {
+    if( prefs ) {
+	for(j=0; prefs[j].type; j++ ) {
+	    if( prefs[j].type == preftype ) {
+                if( (bits[prefs[j].value/32] & (1<<(prefs[j].value%32))) ) {
+		    if( algo_available( preftype, prefs[j].value ) ) {
 			any = 1;
-			i = pref[j+1];
+			i = prefs[j].value;
 			break;
 		    }
 		}
 	    }
 	}
     }
-    if( !pref || !any ) {
+    if( !prefs || !any ) {
 	for(j=0; j < 256; j++ )
 	    if( (bits[j/32] & (1<<(j%32))) ) {
 		if( algo_available( preftype, j ) ) {
@@ -1148,8 +1036,32 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	    i = 1;  /* yep; we can use compression algo 1 */
     }
 
-    m_free(pref);
     return i;
 }
 
+/*
+ * Select the MDC flag from the pk_list.  We can only use MDC if all recipients
+ * support this feature 
+ */
+int
+select_mdc_from_pklist (PK_LIST pk_list)
+{
+    PK_LIST pkr;
+
+    if( !pk_list )
+	return 0;
+
+    for (pkr = pk_list; pkr; pkr = pkr->next) {
+        int mdc;
+
+        if (pkr->pk->user_id) /* selected by user ID */
+            mdc = pkr->pk->user_id->mdc_feature;
+        else
+            mdc = pkr->pk->mdc_feature;
+        if (!mdc)
+            return 0; /* at least on recipeint does not support it */
+    }
+    return 1; /* can be used */
+}
+
 
