@@ -1,5 +1,5 @@
 /* revoke.c
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -87,41 +87,41 @@ gen_revoke( const char *uname )
     IOBUF out = NULL;
     KBNODE keyblock = NULL;
     KBNODE node;
-    KBPOS kbpos;
+    KEYDB_HANDLE kdbhd;
     struct revocation_reason_info *reason = NULL;
+    KEYDB_SEARCH_DESC desc;
 
     if( opt.batch ) {
 	log_error(_("sorry, can't do this in batch mode\n"));
 	return G10ERR_GENERAL;
     }
 
-
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
     init_packet( &pkt );
 
-
-    /* search the userid */
-    rc = find_secret_keyblock_byname( &kbpos, uname );
-    if( rc ) {
-	log_error(_("secret key for user `%s' not found\n"), uname );
+    /* search the userid: 
+     * We don't want the whole getkey stuff here but the entire keyblock
+     */
+    kdbhd = keydb_new (1);
+    classify_user_id (uname, &desc);
+    rc = desc.mode? keydb_search (kdbhd, &desc, 1) : G10ERR_INV_USER_ID;
+    if (rc) {
+	log_error (_("secret key `%s' not found: %s\n"),
+                   uname, g10_errstr (rc));
 	goto leave;
     }
 
-    /* read the keyblock */
-    rc = read_keyblock( &kbpos, &keyblock );
+    rc = keydb_get_keyblock (kdbhd, &keyblock );
     if( rc ) {
-	log_error(_("error reading the certificate: %s\n"), g10_errstr(rc) );
+	log_error (_("error reading keyblock: %s\n"), g10_errstr(rc) );
 	goto leave;
     }
 
     /* get the keyid from the keyblock */
     node = find_kbnode( keyblock, PKT_SECRET_KEY );
-    if( !node ) { /* maybe better to use log_bug ? */
-	log_error(_("Oops; secret key not found anymore!\n"));
-	rc = G10ERR_GENERAL;
-	goto leave;
-    }
+    if( !node ) 
+	BUG ();
 
     /* fixme: should make a function out of this stuff,
      * it's used all over the source */
@@ -130,7 +130,7 @@ gen_revoke( const char *uname )
     tty_printf("\nsec  %4u%c/%08lX %s   ",
 	      nbits_from_sk( sk ),
 	      pubkey_letter( sk->pubkey_algo ),
-	      sk_keyid[1], datestr_from_sk(sk) );
+	      (ulong)sk_keyid[1], datestr_from_sk(sk) );
     {
 	size_t n;
 	char *p = get_user_id( sk_keyid, &n );
@@ -139,6 +139,8 @@ gen_revoke( const char *uname )
 	tty_printf("\n");
     }
     pk = m_alloc_clear( sizeof *pk );
+
+    /* FIXME: We should get the public key direct from the secret one */
     rc = get_pubkey( pk, sk_keyid );
     if( rc ) {
 	log_error(_("no corresponding public key: %s\n"), g10_errstr(rc) );
@@ -191,7 +193,7 @@ gen_revoke( const char *uname )
     iobuf_push_filter( out, armor_filter, &afx );
 
     /* create it */
-    rc = make_keysig_packet( &sig, pk, NULL, NULL, sk, 0x20, 0,
+    rc = make_keysig_packet( &sig, pk, NULL, NULL, sk, 0x20, 0, 0,
 						  revocation_reason_build_cb,
 						  reason );
     if( rc ) {
@@ -224,6 +226,7 @@ gen_revoke( const char *uname )
     if( sig )
 	free_seckey_enc( sig );
     release_kbnode( keyblock );
+    keydb_release (kdbhd);
     if( rc )
 	iobuf_cancel(out);
     else
@@ -243,7 +246,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
     const char *text_1 = _("Key has been compromised");
     const char *text_2 = _("Key is superseded");
     const char *text_3 = _("Key is no longer used");
-    const char *text_4 = _("User ID is non longer valid");
+    const char *text_4 = _("User ID is no longer valid");
     const char *code_text = NULL;
 
     do {
@@ -293,7 +296,7 @@ ask_revocation_reason( int key_rev, int cert_rev, int hint )
 		code_text = text_3;
 	    }
 	    else if( cert_rev && n == 4 ) {
-		code = 0x20; /* uid is non longer valid */
+		code = 0x20; /* uid is no longer valid */
 		code_text = text_4;
 	    }
 	    else
