@@ -1,5 +1,5 @@
 /* ringedit.c -  Function for key ring editing
- *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -198,7 +198,7 @@ add_keyblock_resource( const char *url, int force, int secret )
     char *filename = NULL;
     int rc = 0;
     enum resource_type rt = rt_UNKNOWN;
-
+    const char *created_fname = NULL;
 
     /* Do we have an URL?
      *	gnupg-gdbm:filename  := this is a GDBM resource
@@ -317,12 +317,14 @@ add_keyblock_resource( const char *url, int force, int secret )
 	      #endif
 		if( !opt.quiet )
 		    log_info(_("%s: keyring created\n"), filename );
+                created_fname = filename;
 	    }
 	}
       #if HAVE_DOSISH_SYSTEM || 1
 	iobuf_close( iobuf );
 	iobuf = NULL;
-	/* must close it again */
+        if (created_fname) /* must invalidate that ugly cache */
+            iobuf_ioctl (NULL, 2, 0, (char*)created_fname);
       #endif
 	break;
 
@@ -1106,7 +1108,7 @@ keyring_search( PACKET *req, KBPOS *kbpos, IOBUF iobuf, const char *fname )
     int rc;
     PACKET pkt;
     int save_mode;
-    ulong offset;
+    off_t offset;
     int pkttype = req->pkttype;
     PKT_public_key *req_pk = req->pkt.public_key;
     PKT_secret_key *req_sk = req->pkt.secret_key;
@@ -1188,9 +1190,9 @@ keyring_read( KBPOS *kbpos, KBNODE *ret_root )
     }
 
     if( !kbpos->valid )
-	log_debug("kbpos not valid in keyring_read, want %d\n", (int)kbpos->offset );
+       log_debug("kbpos not valid in keyring_read\n" );
     if( iobuf_seek( a, kbpos->offset ) ) {
-	log_error("can't seek to %lu\n", kbpos->offset);
+       log_error("can't seek\n");
 	iobuf_close(a);
 	return G10ERR_KEYRING_OPEN;
     }
@@ -1529,6 +1531,8 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 
     /* rename and make backup file */
     if( !rentry->secret ) {  /* but not for secret keyrings */
+        iobuf_ioctl (NULL, 2, 0, bakfname );
+        iobuf_ioctl (NULL, 2, 0, rentry->fname );
       #ifdef HAVE_DOSISH_SYSTEM
 	remove( bakfname );
       #endif
@@ -1539,6 +1543,8 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	    goto leave;
 	}
     }
+    iobuf_ioctl (NULL, 2, 0, tmpfname );
+    iobuf_ioctl (NULL, 2, 0, rentry->fname );
   #ifdef HAVE_DOSISH_SYSTEM
     remove( rentry->fname );
   #endif
