@@ -1,5 +1,5 @@
 /* sig-check.c -  Check a signature
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -66,9 +66,6 @@ do_signature_check( PKT_signature *sig, MD_HANDLE digest,
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
     int rc=0;
 
-    if( is_RSA(sig->pubkey_algo) )
-	write_status(STATUS_RSA_OR_IDEA);
-
     *r_expiredate = 0;
     if( get_pubkey( pk, sig->keyid ) )
 	rc = G10ERR_NO_PUBKEY;
@@ -156,9 +153,11 @@ mdc_kludge_check( PKT_signature *sig, MD_HANDLE digest )
 	size_t n;
 	md_putc( digest, sig->pubkey_algo );
 	md_putc( digest, sig->digest_algo );
-	if( sig->hashed_data ) {
-	    n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
-	    md_write( digest, sig->hashed_data, n+2 );
+	if( sig->hashed ) {
+	    n = sig->hashed->len;
+            md_putc (digest, (n >> 8) );
+            md_putc (digest,  n       );
+	    md_write (digest, sig->hashed->data, n);
 	    n += 6;
 	}
 	else
@@ -324,8 +323,13 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
     }
 
     if( pk->expiredate && pk->expiredate < cur_time ) {
-	log_info(_("NOTE: signature key expired %s\n"),
-					asctimestamp( pk->expiredate ) );
+        if (opt.verbose) {
+	    u32 tmp_kid[2];
+
+	    keyid_from_pk( pk, tmp_kid );
+            log_info(_("NOTE: signature key %08lX expired %s\n"),
+                     (ulong)tmp_kid[1], asctimestamp( pk->expiredate ) );
+        }
 	write_status(STATUS_SIGEXPIRED);
 	*r_expired = 1;
     }
@@ -355,9 +359,11 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 	size_t n;
 	md_putc( digest, sig->pubkey_algo );
 	md_putc( digest, sig->digest_algo );
-	if( sig->hashed_data ) {
-	    n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
-	    md_write( digest, sig->hashed_data, n+2 );
+	if( sig->hashed ) {
+	    n = sig->hashed->len;
+            md_putc (digest, (n >> 8) );
+            md_putc (digest,  n       );
+	    md_write (digest, sig->hashed->data, n);
 	    n += 6;
 	}
 	else
@@ -435,6 +441,34 @@ hash_uid_node( KBNODE unode, MD_HANDLE md, PKT_signature *sig )
     }
 }
 
+static void
+cache_selfsig_result ( PKT_signature *sig, int result )
+{
+    byte buf[6];
+
+    if ( opt.no_sig_cache )
+        return;
+
+    buf[0] = 'G';
+    buf[1] = 'P';
+    buf[2] = 'G';
+    buf[3] = 0;
+    if ( !result ) {
+        buf[4] = 1; /* mark cache valid */
+        buf[5] = 1; /* mark signature valid */
+    }
+    else if ( result == G10ERR_BAD_SIGN ) {
+        buf[4] = 1; /* mark cache valid */
+        buf[5] = 0; /* mark signature invalid */
+    }
+    else {
+        buf[4] = 0; /* mark cache invalid */
+        buf[5] = 0; 
+    }
+
+    build_sig_subpkt (sig, SIGSUBPKT_PRIV_VERIFY_CACHE, buf, 6 );
+}
+
 /****************
  * check the signature pointed to by NODE. This is a key signature.
  * If the function detects a self-signature, it uses the PK from
@@ -475,13 +509,33 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 		      sig->flags.valid? "good":"bad" );
   #endif
 
+    /* Check whether we have cached the result of a previous signature check.*/
+    if ( !opt.no_sig_cache ) {
+        const byte *p;
+        size_t len;
+
+        p = parse_sig_subpkt( sig->unhashed,
+                              SIGSUBPKT_PRIV_VERIFY_CACHE, &len );
+        if ( p && len >= 2 && p[0] == 1 ) { /* cache hit */
+	    if( is_selfsig ) {	
+		u32 keyid[2];	
+
+		keyid_from_pk( pk, keyid );
+		if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
+		    *is_selfsig = 1;
+	    }
+            return p[1] == 1? 0 : G10ERR_BAD_SIGN;
+        }
+    }
+
     if( (rc=check_digest_algo(algo)) )
 	return rc;
 
-    if( sig->sig_class == 0x20 ) {
+    if( sig->sig_class == 0x20 ) { /* key revocation */
 	md = md_open( algo, 0 );
 	hash_public_key( md, pk );
 	rc = do_check( pk, sig, md, r_expired );
+        cache_selfsig_result ( sig, rc );
 	md_close(md);
     }
     else if( sig->sig_class == 0x28 ) { /* subkey revocation */
@@ -492,14 +546,17 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
 	    rc = do_check( pk, sig, md, r_expired );
+            cache_selfsig_result ( sig, rc );
 	    md_close(md);
 	}
 	else {
-	    log_error("no subkey for subkey revocation packet\n");
+            if (!opt.quiet)
+                log_info ("key %08lX: no subkey for subkey revocation packet\n",
+                          (ulong)keyid_from_pk (pk, NULL));
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
-    else if( sig->sig_class == 0x18 ) {
+    else if( sig->sig_class == 0x18 ) { /* key binding */
 	KBNODE snode = find_prev_kbnode( root, node, PKT_PUBLIC_SUBKEY );
 
 	if( snode ) {
@@ -514,14 +571,24 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
 	    rc = do_check( pk, sig, md, r_expired );
+            cache_selfsig_result ( sig, rc );
 	    md_close(md);
 	}
 	else {
-	    log_error("no subkey for key signature packet\n");
+            if (!opt.quiet)
+                log_info ("key %08lX: no subkey for subkey binding packet\n",
+                          (ulong)keyid_from_pk (pk, NULL));
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
-    else {
+    else if( sig->sig_class == 0x1f ) { /* direct key signature */
+	md = md_open( algo, 0 );
+	hash_public_key( md, pk );
+	rc = do_check( pk, sig, md, r_expired );
+        cache_selfsig_result ( sig, rc );
+	md_close(md);
+    }
+    else { /* all other classes */
 	KBNODE unode = find_prev_kbnode( root, node, PKT_USER_ID );
 
 	if( unode ) {
@@ -535,6 +602,7 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 		if( is_selfsig )
 		    *is_selfsig = 1;
 		rc = do_check( pk, sig, md, r_expired );
+                cache_selfsig_result ( sig, rc );
 	    }
 	    else {
 		rc = do_signature_check( sig, md, r_expiredate, r_expired );
@@ -542,7 +610,10 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 	    md_close(md);
 	}
 	else {
-	    log_error("no user ID for key signature packet\n");
+            if (!opt.quiet)
+                log_info ("key %08lX: no user ID for key signature packet "
+                          "of class %02x\n",
+                          (ulong)keyid_from_pk (pk, NULL), sig->sig_class );
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
