@@ -1,5 +1,5 @@
 /* sign.c - sign data
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -132,6 +132,27 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 			     digest_algo, mpi_get_nbits(sk->skey[0]), 0 );
     rc = pubkey_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
     mpi_free(frame);
+    if (!rc && !opt.no_sig_create_check) {
+        /* check that the signature verification worked and nothing is
+         * fooling us e.g. by a bug in the signature create
+         * code or by deliberately introduced faults. */
+        PKT_public_key *pk = m_alloc_clear (sizeof *pk);
+
+        if( get_pubkey( pk, sig->keyid ) )
+            rc = G10ERR_NO_PUBKEY;
+        else {
+            frame = encode_md_value (pk->pubkey_algo, md,
+                                     sig->digest_algo,
+                                     mpi_get_nbits(pk->pkey[0]), 0);
+            rc = pubkey_verify (pk->pubkey_algo, frame, sig->data, pk->pkey,
+                                NULL, NULL );
+            mpi_free (frame);
+        }
+        if (rc)
+            log_error (_("checking created signature failed: %s\n"),
+                         g10_errstr (rc));
+        free_public_key (pk);
+    }
     if( rc )
 	log_error(_("signing failed: %s\n"), g10_errstr(rc) );
     else {
@@ -154,10 +175,6 @@ complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md )
 
     if( !(rc=check_secret_key( sk, 0 )) )
 	rc = do_sign( sk, sig, md, 0 );
-
-    /* fixme: should we check whether the signature is okay?
-     * maybe by using an option */
-
     return rc;
 }
 
@@ -430,7 +447,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    if( !(filesize = iobuf_get_filelength(inp)) )
 		log_info(_("WARNING: `%s' is an empty file\n"), fname );
             /* we can't yet encode the length of very large files,
-             * so we switch to partial lengthn encoding in this case */
+             * so we switch to partial length encoding in this case */
             if ( filesize >= IOBUF_FILELENGTH_LIMIT )
                 filesize = 0;
 
@@ -562,8 +579,11 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
   leave:
     if( rc )
 	iobuf_cancel(out);
-    else
+    else {
 	iobuf_close(out);
+        if (encrypt)
+            write_status( STATUS_END_ENCRYPTION );
+    }
     iobuf_close(inp);
     md_close( mfx.md );
     release_sk_list( sk_list );
@@ -630,7 +650,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     }
 
     if( old_style && only_md5 )
-	iobuf_writestr(out, "\n" );
+	iobuf_writestr(out, LF );
     else {
 	const char *s;
 	int any = 0;
@@ -654,11 +674,11 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	    }
 	}
 	assert(any);
-	iobuf_writestr(out, "\n" );
+	iobuf_writestr(out, LF );
 	if( opt.not_dash_escaped )
 	    iobuf_writestr( out,
-		"NotDashEscaped: You need GnuPG to verify this message\n" );
-	iobuf_writestr(out, "\n" );
+		"NotDashEscaped: You need GnuPG to verify this message" LF );
+	iobuf_writestr(out, LF );
     }
 
 
