@@ -1,5 +1,5 @@
 /* trustdb.c
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -467,20 +467,20 @@ lid_from_keyid_no_sdir( u32 *keyid )
 void
 register_trusted_key( const char *string )
 {
-    u32 keyid[2];
+    KEYDB_SEARCH_DESC desc;
     struct keyid_list *r;
 
-    if( classify_user_id( string, keyid, NULL, NULL, NULL ) != 11 ) {
-        log_error(_("'%s' is not a valid long keyID\n"), string );
+    if (classify_user_id (string, &desc) != KEYDB_SEARCH_MODE_LONG_KID ) {
+        log_error(_("`%s' is not a valid long keyID\n"), string );
         return;
     }
 
     for( r = trusted_key_list; r; r = r->next )
-        if( r->keyid[0] == keyid[0] && r->keyid[1] == keyid[1] )
+        if( r->keyid[0] == desc.u.kid[0] && r->keyid[1] == desc.u.kid[1] )
             return;
     r = m_alloc( sizeof *r );
-    r->keyid[0] = keyid[0];
-    r->keyid[1] = keyid[1];
+    r->keyid[0] = desc.u.kid[0];
+    r->keyid[1] = desc.u.kid[1];
     r->next = trusted_key_list;
     trusted_key_list = r;
 }
@@ -513,7 +513,7 @@ add_ultimate_key( PKT_public_key *pk, u32 *keyid )
                   (ulong)keyid[1], pk->local_id );
 
     if( ins_lid_table_item( ultikey_table, pk->local_id, 0 ) )
-        log_error(_("key %08lX: already in trusted key table\n"),
+        log_info(_("key %08lX: already in trusted key table\n"),
                   (ulong)keyid[1]);
     else if( opt.verbose > 1 )
         log_info(_("key %08lX: accepted as trusted key.\n"),
@@ -574,8 +574,9 @@ verify_own_keys(void)
 	memset( pk, 0, sizeof *pk );
 	rc = get_pubkey( pk, keyid );
 	if( rc ) {
-	    log_info(_("key %08lX: secret key without public key - skipped\n"),
-							    (ulong)keyid[1] );
+            if (!opt.quiet)
+                log_info(_("key %08lX: secret key without public key "
+                           "- skipped\n"),  (ulong)keyid[1] );
 	    goto skip;
 	}
 	have_pk=1;
@@ -1377,72 +1378,6 @@ make_sig_records( KBNODE keyblock, KBNODE uidnode,
 
 
 
-/****************
- * Make a preference record (or a list of them) according to the supplied
- * signature.
- * Returns: The record number of the first pref record.
- */
-static ulong
-make_pref_record( PKT_signature *sig, ulong lid )
-{
-    static struct {
-	sigsubpkttype_t subpkttype;
-	int preftype;
-    } ptable[] = {
-	{ SIGSUBPKT_PREF_SYM,	PREFTYPE_SYM	},
-	{ SIGSUBPKT_PREF_HASH,	PREFTYPE_HASH	},
-	{ SIGSUBPKT_PREF_COMPR, PREFTYPE_COMPR	},
-	{ 0, 0 }
-    };
-    TRUSTREC *precs, **p_end, *p=NULL, *p2;
-    ulong precno;
-    int k, idx=0;
-    const byte *s;
-    size_t n;
-
-  #if (ITEMS_PER_PREF_RECORD % 2) != 0
-    #error ITEMS_PER_PREF_RECORD must have an even value
-  #endif
-
-    precs = NULL; p_end = &precs;
-    for(k=0; ptable[k].subpkttype; k++ ) {
-	s = parse_sig_subpkt2( sig, ptable[k].subpkttype, &n );
-	if( !s )
-	    continue;
-	for( ; n; n--, s++ ) {
-	    if( !idx ) {
-		p = m_alloc_clear( sizeof *p );
-		p->rectype = RECTYPE_PREF;
-		p->r.pref.lid = lid;
-	    }
-	    p->r.pref.data[idx++] = ptable[k].preftype;
-	    p->r.pref.data[idx++] = *s;
-	    if( idx >= ITEMS_PER_PREF_RECORD ) {
-		p->recnum = tdbio_new_recnum();
-		*p_end = p;
-		p_end = &p->next;
-		idx = 0;
-	    }
-	}
-    }
-    if( idx ) {
-       p->recnum = tdbio_new_recnum();
-       *p_end = p;
-       p_end = &p->next;
-    }
-
-    precno = precs? precs->recnum : 0;
-    /* write the precs and release the memory */
-    for( p = precs; p ; p = p2 ) {
-	if( p->next )
-	    p->r.pref.next = p->next->recnum;
-	write_record( p );
-	p2 = p->next;
-	m_free( p );
-    }
-    return precno;
-}
-
 
 static ulong
 make_uid_records( KBNODE keyblock, ulong lid, u32 *keyid, u32 *min_expire,
@@ -1479,7 +1414,7 @@ make_uid_records( KBNODE keyblock, ulong lid, u32 *keyid, u32 *min_expire,
 						     lid, &bestsig );
 	if( (u->r.uid.uidflags & UIDF_CHECKED)
 	    && (u->r.uid.uidflags & UIDF_VALID) ) {
-	    u->r.uid.prefrec = bestsig? make_pref_record( bestsig, lid ) : 0;
+	    u->r.uid.prefrec = 0;
 	}
 
 	/* the next test is really bad because we should modify
@@ -1578,6 +1513,8 @@ do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
     for( recno=drec->r.dir.uidlist; recno; recno = urec.r.uid.next ) {
 	read_record( recno, &urec, RECTYPE_UID );
 	for(r2=urec.r.uid.prefrec ; r2; r2 = prec.r.pref.next ) {
+            /* we don't use preference records any more, but all ones might
+             * still be there */
 	    read_record( r2, &prec, RECTYPE_PREF );
 	    delete_record( r2 );
 	}
@@ -1760,22 +1697,28 @@ void
 update_trustdb()
 {
     KBNODE keyblock = NULL;
-    KBPOS kbpos;
+    KEYDB_HANDLE kdbhd;
     int rc;
 
     if( opt.dry_run )
 	return;
 
     init_trustdb();
-    rc = enum_keyblocks( 0, &kbpos, &keyblock );
-    if( !rc ) {
+    kdbhd = keydb_new (0);
+    rc = keydb_search_first (kdbhd);
+    if (!rc) {
 	ulong count=0, err_count=0, new_count=0;
 
-	while( !(rc = enum_keyblocks( 1, &kbpos, &keyblock )) ) {
-	    /*int modified;*/
+	do {
 	    TRUSTREC drec;
-	    PKT_public_key *pk = find_kbnode( keyblock, PKT_PUBLIC_KEY )
-					->pkt->pkt.public_key;
+	    PKT_public_key *pk;
+	    /*int modified;*/
+
+            rc = keydb_get_keyblock (kdbhd, &keyblock );
+            if (rc)
+                break;
+
+	    pk = find_kbnode (keyblock, PKT_PUBLIC_KEY)->pkt->pkt.public_key;
 
 	    rc = get_dir_record( pk, &drec );
 	    if( rc == -1 ) { /* not in trustdb: insert */
@@ -1804,7 +1747,7 @@ update_trustdb()
 	    release_kbnode( keyblock ); keyblock = NULL;
 	    if( !(++count % 100) )
 		log_info(_("%lu keys so far processed\n"), count);
-	}
+	} while ( !(rc = keydb_search_next (kdbhd)));
 	log_info(_("%lu keys processed\n"), count);
 	if( err_count )
 	    log_info(_("\t%lu keys with errors\n"), err_count);
@@ -1814,7 +1757,7 @@ update_trustdb()
     if( rc && rc != -1 )
 	log_error(_("enumerate keyblocks failed: %s\n"), g10_errstr(rc));
 
-    enum_keyblocks( 2, &kbpos, &keyblock ); /* close */
+    keydb_release (kdbhd);
     release_kbnode( keyblock );
 }
 
@@ -2364,11 +2307,11 @@ query_trust_record( PKT_public_key *pk )
  *	     is not necessary to check this if we use a local pubring. Hmmmm.
  */
 int
-check_trust( PKT_public_key *pk, unsigned *r_trustlevel,
+check_trust( PKT_public_key *pk, int *r_trustlevel,
 	     const byte *namehash, int (*add_fnc)(ulong), unsigned *retflgs )
 {
     TRUSTREC rec;
-    unsigned trustlevel = TRUST_UNKNOWN;
+    unsigned int trustlevel = TRUST_UNKNOWN;
     int rc=0;
     u32 cur_time;
     u32 keyid[2];
@@ -2520,7 +2463,7 @@ mark_fresh_keys()
 int
 query_trust_info( PKT_public_key *pk, const byte *namehash )
 {
-    unsigned trustlevel;
+    int trustlevel;
     int c;
 
     init_trustdb();
@@ -2577,7 +2520,7 @@ list_trust_path( const char *username )
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
 
     init_trustdb();
-    if( (rc = get_pubkey_byname(NULL, pk, username, NULL )) )
+    if( (rc = get_pubkey_byname (pk, username, NULL, NULL )) )
 	log_error(_("user '%s' not found: %s\n"), username, g10_errstr(rc) );
     else if( (rc=tdbio_search_dir_bypk( pk, &rec )) && rc != -1 )
 	log_error(_("problem finding '%s' in trustdb: %s\n"),
@@ -2747,69 +2690,3 @@ enum_cert_paths_print( void **context, FILE *fp,
 }
 
 
-/*
- * Return an allocated buffer with the preference values for
- * the key with LID and the userid which is identified by the
- * HAMEHASH or the first one if namehash is NULL.  ret_n receives
- * the length of the allocated buffer.	Structure of the buffer is
- * a repeated sequences of 2 bytes; where the first byte describes the
- * type of the preference and the second one the value.  The constants
- * PREFTYPE_xxxx should be used to reference a type.
- */
-byte *
-get_pref_data( ulong lid, const byte *namehash, size_t *ret_n )
-{
-    TRUSTREC rec;
-    ulong recno;
-
-    init_trustdb();
-    read_record( lid, &rec, RECTYPE_DIR );
-    for( recno=rec.r.dir.uidlist; recno; recno = rec.r.uid.next ) {
-	read_record( recno, &rec, RECTYPE_UID );
-	if( rec.r.uid.prefrec
-	    && ( !namehash || !memcmp(namehash, rec.r.uid.namehash, 20) ))  {
-	    byte *buf;
-	    /* found the correct one or the first one */
-	    read_record( rec.r.uid.prefrec, &rec, RECTYPE_PREF );
-	    if( rec.r.pref.next )
-		log_info(_("WARNING: can't yet handle long pref records\n"));
-	    buf = m_alloc( ITEMS_PER_PREF_RECORD );
-	    memcpy( buf, rec.r.pref.data, ITEMS_PER_PREF_RECORD );
-	    *ret_n = ITEMS_PER_PREF_RECORD;
-	    return buf;
-	}
-    }
-    return NULL;
-}
-
-
-
-/****************
- * Check whether the algorithm is in one of the pref records
- */
-int
-is_algo_in_prefs( ulong lid, int preftype, int algo )
-{
-    TRUSTREC rec;
-    ulong recno;
-    int i;
-    byte *pref;
-
-    init_trustdb();
-    read_record( lid, &rec, RECTYPE_DIR );
-    for( recno=rec.r.dir.uidlist; recno; recno = rec.r.uid.next ) {
-	read_record( recno, &rec, RECTYPE_UID );
-	if( rec.r.uid.prefrec ) {
-	    read_record( rec.r.uid.prefrec, &rec, RECTYPE_PREF );
-	    if( rec.r.pref.next )
-		log_info(_("WARNING: can't yet handle long pref records\n"));
-	    pref = rec.r.pref.data;
-	    for(i=0; i+1 < ITEMS_PER_PREF_RECORD; i+=2 ) {
-		if( pref[i] == preftype && pref[i+1] == algo )
-		    return 1;
-	    }
-	}
-    }
-    return 0;
-}
-
