@@ -1,5 +1,5 @@
 /* cipher.h
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -33,6 +33,9 @@
 #define CIPHER_ALGO_BLOWFISH	 4  /* blowfish 128 bit key */
 #define CIPHER_ALGO_SAFER_SK128  5
 #define CIPHER_ALGO_DES_SK	 6
+#define CIPHER_ALGO_RIJNDAEL     7
+#define CIPHER_ALGO_RIJNDAEL192  8
+#define CIPHER_ALGO_RIJNDAEL256  9
 #define CIPHER_ALGO_TWOFISH	10  /* twofish 256 bit */
 #define CIPHER_ALGO_SKIPJACK   101  /* experimental: skipjack */
 #define CIPHER_ALGO_TWOFISH_OLD 102 /* experimental: twofish 128 bit */
@@ -60,6 +63,8 @@
 typedef struct {
     int algo;
     int keylen;
+    int algo_info_printed;
+    int use_mdc;
     byte key[32]; /* this is the largest used keylen (256 bit) */
 } DEK;
 
@@ -88,10 +93,17 @@ struct gcry_md_context {
 
 typedef struct gcry_md_context *MD_HANDLE;
 
+#ifndef EXTERN_UNLESS_MAIN_MODULE
+ #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
+  #define EXTERN_UNLESS_MAIN_MODULE extern
+ #else
+  #define EXTERN_UNLESS_MAIN_MODULE 
+ #endif
+#endif
+EXTERN_UNLESS_MAIN_MODULE int g10c_debug_mode;
+EXTERN_UNLESS_MAIN_MODULE int g10_opt_verbose;
+EXTERN_UNLESS_MAIN_MODULE const char *g10_opt_homedir;
 
-int g10c_debug_mode;
-int g10_opt_verbose;
-const char *g10_opt_homedir;
 
 /*-- dynload.c --*/
 void register_cipher_extension( const char *mainpgm, const char *fname );
@@ -105,7 +117,7 @@ void md_enable( MD_HANDLE hd, int algo );
 MD_HANDLE md_copy( MD_HANDLE a );
 void md_reset( MD_HANDLE a );
 void md_close(MD_HANDLE a);
-void md_write( MD_HANDLE a, byte *inbuf, size_t inlen);
+void md_write( MD_HANDLE a, const byte *inbuf, size_t inlen);
 void md_final(MD_HANDLE a);
 byte *md_read( MD_HANDLE a, int algo );
 int md_digest( MD_HANDLE a, int algo, byte *buffer, int buflen );
@@ -128,6 +140,7 @@ void rmd160_hash_buffer( char *outbuf, const char *buffer, size_t length );
 /*-- cipher.c --*/
 int string_to_cipher_algo( const char *string );
 const char * cipher_algo_to_string( int algo );
+void disable_cipher_algo( int algo );
 int check_cipher_algo( int algo );
 unsigned cipher_get_keylen( int algo );
 unsigned cipher_get_blocksize( int algo );
@@ -147,6 +160,7 @@ void cipher_sync( CIPHER_HANDLE c );
 
 int string_to_pubkey_algo( const char *string );
 const char * pubkey_algo_to_string( int algo );
+void disable_pubkey_algo( int algo );
 int check_pubkey_algo( int algo );
 int check_pubkey_algo2( int algo, unsigned use );
 int pubkey_get_npkey( int algo );
@@ -166,10 +180,14 @@ int pubkey_verify( int algo, MPI hash, MPI *data, MPI *pkey,
 extern ushort small_prime_numbers[];
 
 /*-- primegen.c --*/
+void register_primegen_progress ( void (*cb)( void *, int), void *cb_data );
 MPI generate_secret_prime( unsigned nbits );
 MPI generate_public_prime( unsigned nbits );
 MPI generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 					   MPI g, MPI **factors );
 
+/*-- elsewhere --*/
+void register_pk_dsa_progress ( void (*cb)( void *, int), void *cb_data );
+void register_pk_elg_progress ( void (*cb)( void *, int), void *cb_data );
 
 #endif /*G10_CIPHER_H*/
