@@ -1,5 +1,5 @@
 /* errors.h - erro code
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -71,6 +71,10 @@
 #define G10ERR_UNKNOWN_HOST   49
 #define G10ERR_SELFTEST_FAILED 50
 #define G10ERR_NOT_ENCRYPTED  51
+#define G10ERR_NOT_PROCESSED  52
+#define G10ERR_UNU_PUBKEY     53
+#define G10ERR_UNU_SECKEY     54
+
 
 #ifndef HAVE_STRERROR
 char *strerror( int n );
