@@ -1,5 +1,5 @@
 /* http.h  -  HTTP protocol handler
- *	Copyright (C) 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -49,6 +49,11 @@ typedef enum {
     HTTP_REQ_POST = 3
 } HTTP_REQ_TYPE;
 
+enum {	/* put flag values into an enum, so that gdb can display them */
+    HTTP_FLAG_TRY_PROXY = 1,
+    HTTP_FLAG_NO_SHUTDOWN = 2,
+};
+
 struct http_context {
     int initialized;
     unsigned int status_code;
@@ -61,6 +66,7 @@ struct http_context {
     HTTP_REQ_TYPE req_type;
     byte *buffer;	   /* line buffer */
     unsigned buffer_size;
+    unsigned int flags;
 };
 typedef struct http_context *HTTP_HD;
 
