@@ -1,5 +1,6 @@
 /* mpi.h  -  Multi Precision Integers
- *	Copyright (C) 1994, 1996, 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1994, 1996, 1998, 1999,
+ *                    2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -29,20 +30,33 @@
 #ifndef G10_MPI_H
 #define G10_MPI_H
 
+#include <config.h>
 #include <stdio.h>
 #include "iobuf.h"
 #include "types.h"
 #include "memory.h"
 
+#if BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_INT
+  typedef unsigned int mpi_limb_t;
+  typedef   signed int mpi_limb_signed_t;
+#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_LONG
+  typedef unsigned long int mpi_limb_t;
+  typedef   signed long int mpi_limb_signed_t;
+#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_LONG_LONG
+  typedef unsigned long long int mpi_limb_t;
+  typedef   signed long long int mpi_limb_signed_t;
+#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_SHORT
+  typedef unsigned short int mpi_limb_t;
+  typedef   signed short int mpi_limb_signed_t;
+#else
+  #error BYTES_PER_MPI_LIMB does not match any C type
+#endif
+#define BITS_PER_MPI_LIMB    (8*BYTES_PER_MPI_LIMB)
+
 
 #define DBG_MPI     mpi_debug_mode
 int mpi_debug_mode;
 
-#define BITS_PER_MPI_LIMB    (8*SIZEOF_UNSIGNED_LONG)
-#define BYTES_PER_MPI_LIMB   SIZEOF_UNSIGNED_LONG
-typedef unsigned long int mpi_limb_t;
-typedef   signed long int mpi_limb_signed_t;
-
 struct gcry_mpi {
     int alloced;    /* array size (# of allocated limbs) */
     int nlimbs;     /* number of valid limbs */
@@ -110,11 +124,14 @@ int mpi_write( IOBUF out, MPI a );
 MPI mpi_read_from_buffer(byte *buffer, unsigned *ret_nread, int secure);
 int mpi_fromstr(MPI val, const char *str);
 int mpi_print( FILE *fp, MPI a, int mode );
+void g10_log_mpidump( const char *text, MPI a );
 u32 mpi_get_keyid( MPI a, u32 *keyid );
 byte *mpi_get_buffer( MPI a, unsigned *nbytes, int *sign );
 byte *mpi_get_secure_buffer( MPI a, unsigned *nbytes, int *sign );
 void  mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign );
 
+#define log_mpidump g10_log_mpidump
+
 /*-- mpi-add.c --*/
 void mpi_add_ui(MPI w, MPI u, ulong v );
 void mpi_add(MPI w, MPI u, MPI v);
