@@ -1,13 +1,75 @@
 #!/bin/sh
 # Run this to generate all the initial makefiles, etc.
+#
+# Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+#
+# This file is free software; as a special exception the author gives
+# unlimited permission to copy and/or distribute it, with or without
+# modifications, as long as this notice is preserved.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 PGM=GnuPG
+lib_config_files=""
+autoconf_vers=2.52
+automake_vers=1.5
+aclocal_vers=1.5
+
+
 DIE=no
+if test "$1" = "--build-w32"; then
+    tsdir=$(cd `dirname $0`; cd ..; pwd)
+    shift
+    host=i386--mingw32
+    if [ ! -f $tsdir/scripts/config.guess ]; then
+        echo "$tsdir/scripts/config.guess not found" >&2
+        exit 1
+    fi
+    build=`$tsdir/scripts/config.guess`
+        
+    if ! mingw32 --version >/dev/null; then
+        echo "We need at least version 0.3 of MingW32/CPD" >&2
+        exit 1
+    fi
+
+    if [ -f "$tsdir/config.log" ]; then
+        if ! head $tsdir/config.log | grep i386--mingw32 >/dev/null; then
+            echo "Pease run a 'make distclean' first" >&2
+            exit 1
+        fi
+    fi
+
+    crossbindir=`mingw32 --install-dir`/bin
+    CC=`mingw32 --get-path gcc`
+    CPP=`mingw32 --get-path cpp`
+    AR=`mingw32 --get-path ar`
+    RANLIB=`mingw32 --get-path ranlib`
+    export CC CPP AR RANLIB 
+
+    disable_foo_tests=""
+    if [ -n "$lib_config_files" ]; then
+        for i in $lib_config_files; do
+            j=`echo $i | tr '[a-z-]' '[A-Z_]'`
+            eval "$j=${crossbindir}/$i"
+            export $j
+            disable_foo_tests="$disable_foo_tests --disable-`echo $i| \
+                           sed 's,-config$,,'`-test"
+            if [ ! -f "${crossbindir}/$i" ]; then                   
+                echo "$i not installed for MingW32" >&2
+                DIE=yes
+            fi
+        done
+    fi
+    [ $DIE = yes ] && exit 1
+
+    $tsdir/configure --build=${build} --host=${host} \
+                ${disable_foo_tests} $*
+    exit $?
+fi
+
 
-autoconf_vers=2.13
-automake_vers=1.4
-aclocal_vers=1.4
-libtool_vers=1.3
 
 if (autoconf --version) < /dev/null > /dev/null 2>&1 ; then
     if (autoconf --version | awk 'NR==1 { if( $3 >= '$autoconf_vers') \
@@ -55,41 +117,22 @@ fi
 
 
 if (gettext --version </dev/null 2>/dev/null | awk 'NR==1 { split($4,A,"\."); \
-    X=10000*A[1]+100*A[2]+A[3]; echo X; if( X >= 1035 ) exit 1; exit 0}')
+    X=10000*A[1]+100*A[2]+A[3]; echo X; if( X >= 1038 ) exit 1; exit 0}')
     then
     echo "**Error**: You must have "\`gettext\'" installed to compile $PGM."
-    echo '           (version 0.10.35 or newer is required; get'
-    echo '            ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz'
+    echo '           (version 0.10.38 or newer is required; get'
+    echo '            ftp://alpha.gnu.org/gnu/gettext/gettext-0.10.38.tar.gz'
     echo '            or install the latest Debian package)'
     DIE="yes"
 fi
 
 
-if (libtool --version) < /dev/null > /dev/null 2>&1 ; then
-    if (libtool --version | awk 'NR==1 { if( $4 >= '$libtool_vers') \
-			       exit 1; exit 0; }');
-    then
-       echo "**Error**: "\`libtool\'" is too old."
-       echo '           (version ' $libtool_vers ' or newer is required)'
-       DIE="yes"
-    fi
-else
-    echo
-    echo "**Error**: You must have "\`libtool\'" installed to compile $PGM."
-    echo '           (version ' $libtool_vers ' or newer is required)'
-    DIE="yes"
-fi
-
-
 if test "$DIE" = "yes"; then
     exit 1
 fi
 
 echo "Running gettextize...  Ignore non-fatal messages."
 echo "no" | gettextize --force
-echo "Running libtoolize...  Ignore non-fatal messages."
-echo "no" | libtoolize
-
 
 echo "Running aclocal..."
 aclocal
