@@ -1,14 +1,5 @@
 /* memory.c  -	memory allocation
- *	Copyright (C) 1998 Free Software Foundation, Inc.
- *
- * We use our own memory allocation functions instead of plain malloc(),
- * so that we can provide some special enhancements:
- *  a) functions to provide memory from a secure memory.
- *  b) by looking at the requested allocation size we
- *     can reuse memory very quickly (e.g. MPI storage)
- *     (really needed?)
- *  c) memory usage reporting if compiled with M_DEBUG
- *  d) memory checking if compiled with M_GUARD
+ *	Copyright (C) 1998, 1999, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -25,6 +16,16 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ *
+ *
+ * We use our own memory allocation functions instead of plain malloc(),
+ * so that we can provide some special enhancements:
+ *  a) functions to provide memory from a secure memory.
+ *  b) by looking at the requested allocation size we
+ *     can reuse memory very quickly (e.g. MPI storage)
+ *     (really needed?)
+ *  c) memory usage reporting if compiled with M_DEBUG
+ *  d) memory checking if compiled with M_GUARD
  */
 
 #include <config.h>
@@ -42,15 +43,21 @@
 #define MAGIC_SEC_BYTE 0xcc
 #define MAGIC_END_BYTE 0xaa
 
+/* This is a very crude alignment check which does not work on all CPUs
+ * IIRC, I once introduced it for testing on an Alpha.  We should better
+ * replace this guard stuff with one provided by a modern malloc library
+ */
 #if SIZEOF_UNSIGNED_LONG == 8
   #define EXTRA_ALIGN 4
 #else
   #define EXTRA_ALIGN 0
 #endif
 
+#if defined(M_DEBUG) || defined(M_GUARD)
+  static void membug( const char *fmt, ... );
+#endif
 
 #ifdef M_DEBUG
-  static void membug( const char *fmt, ... );
 
   #ifndef M_GUARD
     #define M_GUARD 1
@@ -319,7 +326,9 @@ check_allmem( const char *info )
 	    check_mem(e->user_p-4-EXTRA_ALIGN, info);
 }
 
+#endif /* M_DEBUG */
 
+#if defined(M_DEBUG) || defined(M_GUARD)
 static void
 membug( const char *fmt, ... )
 {
@@ -336,8 +345,7 @@ membug( const char *fmt, ... )
   #endif
     abort();
 }
-
-#endif /* M_DEBUG */
+#endif
 
 void
 m_print_stats( const char *prefix )
@@ -376,9 +384,14 @@ m_dump_table( const char *prefix )
 static void
 out_of_core(size_t n, int secure)
 {
-    log_fatal("out of %s memory while allocating %u bytes\n",
-			secure? "secure":"" ,(unsigned)n );
-
+    log_error ("out of %s memory while allocating %u bytes\n",
+               secure? "secure":"" ,(unsigned)n );
+    if (secure) {
+        /*secmem_dump_stats ();*/
+        log_info ("(this may be caused by too many secret keys used "
+                  "simultaneously or due to excessive large key sizes)\n");
+    }
+    exit (2);
 }
 
 /****************
