@@ -1,5 +1,5 @@
 /* strgutil.c -  string utilities
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ * Copyright (C) 1994, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -65,9 +65,10 @@ static ushort latin2_unicode[128] = {
     0x0159,0x016F,0x00FA,0x0171,0x00FC,0x00FD,0x0163,0x02D9
 };
 
+
 static const char *active_charset_name = "iso-8859-1";
 static ushort *active_charset = NULL;
-
+static int no_translation = 0;
 
 void
 free_strlist( STRLIST sl )
@@ -193,6 +194,25 @@ memistr( const char *buf, size_t buflen, const char *sub )
     return NULL ;
 }
 
+const char *
+ascii_memistr( const char *buf, size_t buflen, const char *sub )
+{
+    const byte *t, *s ;
+    size_t n;
+
+    for( t=buf, n=buflen, s=sub ; n ; t++, n-- )
+	if( ascii_toupper(*t) == ascii_toupper(*s) ) {
+	    for( buf=t++, buflen = n--, s++;
+		 n && ascii_toupper(*t) == ascii_toupper(*s); t++, s++, n-- )
+		;
+	    if( !*s )
+		return buf;
+	    t = buf; n = buflen; s = sub ;
+	}
+
+    return NULL ;
+}
+
 /****************
  * Wie strncpy(), aber es werden maximal n-1 zeichen kopiert und ein
  * '\0' angehngt. Ist n = 0, so geschieht nichts, ist Destination
@@ -247,7 +267,7 @@ trim_spaces( char *str )
 
 
 
-unsigned
+unsigned int
 trim_trailing_chars( byte *line, unsigned len, const char *trimchars )
 {
     byte *p, *mark;
@@ -278,6 +298,37 @@ trim_trailing_ws( byte *line, unsigned len )
     return trim_trailing_chars( line, len, " \t\r\n" );
 }
 
+unsigned int
+check_trailing_chars( const byte *line, unsigned int len,
+                      const char *trimchars )
+{
+    const byte *p, *mark;
+    unsigned int n;
+
+    for(mark=NULL, p=line, n=0; n < len; n++, p++ ) {
+	if( strchr(trimchars, *p ) ) {
+	    if( !mark )
+		mark = p;
+	}
+	else
+	    mark = NULL;
+    }
+
+    if( mark ) {
+	return mark - line;
+    }
+    return len;
+}
+
+/****************
+ * remove trailing white spaces and return the length of the buffer
+ */
+unsigned int
+check_trailing_ws( const byte *line, unsigned int len )
+{
+    return check_trailing_chars( line, len, " \t\r\n" );
+}
+
 
 
 int
@@ -294,18 +345,27 @@ string_count_chr( const char *string, int c )
 int
 set_native_charset( const char *newset )
 {
-    if( !stricmp( newset, "iso-8859-1" ) ) {
+    if( !ascii_strcasecmp( newset, "iso-8859-1" ) ) {
 	active_charset_name = "iso-8859-1";
+        no_translation = 0;
 	active_charset = NULL;
     }
-    else if( !stricmp( newset, "iso-8859-2" ) ) {
+    else if( !ascii_strcasecmp( newset, "iso-8859-2" ) ) {
 	active_charset_name = "iso-8859-2";
+        no_translation = 0;
 	active_charset = latin2_unicode;
     }
-    else if( !stricmp( newset, "koi8-r" ) ) {
+    else if( !ascii_strcasecmp( newset, "koi8-r" ) ) {
 	active_charset_name = "koi8-r";
+        no_translation = 0;
 	active_charset = koi8_unicode;
     }
+    else if( !ascii_strcasecmp (newset, "utf8" )
+             || !ascii_strcasecmp(newset, "utf-8") ) {
+	active_charset_name = "utf-8";
+        no_translation = 1;
+	active_charset = NULL;
+    }
     else
 	return G10ERR_GENERAL;
     return 0;
@@ -329,7 +389,10 @@ native_to_utf8( const char *string )
     byte *p;
     size_t length=0;
 
-    if( active_charset ) {
+    if (no_translation) {
+        buffer = m_strdup (string);
+    }
+    else if( active_charset ) {
 	for(s=string; *s; s++ ) {
 	    length++;
 	    if( *s & 0x80 )
@@ -384,7 +447,7 @@ utf8_to_native( const char *string, size_t length )
 {
     int nleft;
     int i;
-    byte encbuf[7];
+    byte encbuf[8];
     int encidx;
     const byte *s;
     size_t n;
@@ -393,6 +456,13 @@ utf8_to_native( const char *string, size_t length )
     size_t slen;
     int resync = 0;
 
+    if (no_translation) {
+        buffer = m_alloc (length+1);
+        memcpy (buffer, string, length);
+        buffer[length] = 0; /* make sure it is a string */
+        return buffer;
+    }
+
     /* 1. pass (p==NULL): count the extended utf-8 characters */
     /* 2. pass (p!=NULL): create string */
     for( ;; ) {
@@ -422,11 +492,13 @@ utf8_to_native( const char *string, size_t length )
 			  case '\v': n++; if( p ) *p++ = 'v'; break;
 			  case '\b': n++; if( p ) *p++ = 'b'; break;
 			  case	 0 : n++; if( p ) *p++ = '0'; break;
-			  default: n += 3;
-				   sprintf( p, "x%02x", *s );
-                                   if ( p )
-                                       p += 3;
-				   break;
+			  default:
+                            n += 3;
+                            if ( p ) {
+                                sprintf( p, "x%02x", *s );
+                                p += 3;
+                            }
+                            break;
 			}
 		    }
 		    else {
@@ -437,27 +509,32 @@ utf8_to_native( const char *string, size_t length )
 		else if( (*s & 0xe0) == 0xc0 ) { /* 110x xxxx */
 		    val = *s & 0x1f;
 		    nleft = 1;
-		    encbuf[encidx=0] = *s;
+                    encidx = 0;
+		    encbuf[encidx++] = *s;
 		}
 		else if( (*s & 0xf0) == 0xe0 ) { /* 1110 xxxx */
 		    val = *s & 0x0f;
 		    nleft = 2;
-		    encbuf[encidx=0] = *s;
+                    encidx = 0;
+		    encbuf[encidx++] = *s;
 		}
 		else if( (*s & 0xf8) == 0xf0 ) { /* 1111 0xxx */
 		    val = *s & 0x07;
 		    nleft = 3;
-		    encbuf[encidx=0] = *s;
+                    encidx = 0;
+		    encbuf[encidx++] = *s;
 		}
 		else if( (*s & 0xfc) == 0xf8 ) { /* 1111 10xx */
 		    val = *s & 0x03;
 		    nleft = 4;
-		    encbuf[encidx=0] = *s;
+                    encidx = 0;
+		    encbuf[encidx++] = *s;
 		}
 		else if( (*s & 0xfe) == 0xfc ) { /* 1111 110x */
 		    val = *s & 0x01;
 		    nleft = 5;
-		    encbuf[encidx=0] = *s;
+                    encidx = 0;
+		    encbuf[encidx++] = *s;
 		}
 		else {	/* invalid encoding: print as \xnn */
 		    if( p ) {
@@ -470,15 +547,20 @@ utf8_to_native( const char *string, size_t length )
 	    }
 	    else if( *s < 0x80 || *s >= 0xc0 ) { /* invalid */
 		if( p ) {
+                    for(i=0; i < encidx; i++ ) {
+                        sprintf(p, "\\x%02x", encbuf[i] );
+                        p += 4;
+                    }
 		    sprintf(p, "\\x%02x", *s );
 		    p += 4;
 		}
-		n += 4;
+		n += 4 + 4*encidx;
 		nleft = 0;
+                encidx = 0;
 		resync = 1;
 	    }
 	    else {
-		encbuf[++encidx] = *s;
+		encbuf[encidx++] = *s;
 		val <<= 6;
 		val |= *s & 0x3f;
 		if( !--nleft ) { /* ready */
@@ -499,6 +581,7 @@ utf8_to_native( const char *string, size_t length )
 				}
 			    }
 			    n += encidx*4;
+                            encidx = 0;
 			}
 		    }
 		    else { /* native set */
@@ -514,9 +597,9 @@ utf8_to_native( const char *string, size_t length )
 				}
 			    }
 			    n += encidx*4;
+                            encidx = 0;
 			}
 		    }
-
 		}
 
 	    }
@@ -531,6 +614,63 @@ utf8_to_native( const char *string, size_t length )
     }
 }
 
+/****************************************************
+ ******** locale insensitive ctype functions ********
+ ****************************************************/
+/* FIXME: replace them by a table lookup and macros */
+int
+ascii_isupper (int c)
+{
+    return c >= 'A' && c <= 'Z';
+}
+
+int
+ascii_islower (int c)
+{
+    return c >= 'a' && c <= 'z';
+}
+
+int 
+ascii_toupper (int c)
+{
+    if (c >= 'a' && c <= 'z')
+        c &= ~0x20;
+    return c;
+}
+
+int 
+ascii_tolower (int c)
+{
+    if (c >= 'A' && c <= 'Z')
+        c |= 0x20;
+    return c;
+}
+
+
+int
+ascii_strcasecmp( const char *a, const char *b )
+{
+    if (a == b)
+        return 0;
+
+    for (; *a && *b; a++, b++) {
+	if (*a != *b && ascii_toupper(*a) != ascii_toupper(*b))
+	    break;
+    }
+    return *a == *b? 0 : (ascii_toupper (*a) - ascii_toupper (*b));
+}
+
+int
+ascii_memcasecmp( const char *a, const char *b, size_t n )
+{
+    if (a == b)
+        return 0;
+    for ( ; n; n--, a++, b++ ) {
+	if( *a != *b  && ascii_toupper (*a) != ascii_toupper (*b) )
+            return *a == *b? 0 : (ascii_toupper (*a) - ascii_toupper (*b));
+    }
+    return 0;
+}
 
 
 /*********************************************
@@ -587,3 +727,100 @@ memicmp( const char *a, const char *b, size_t n )
 #endif
 
 
+#ifdef __MINGW32__
+/* 
+ * Like vsprintf but provides a pointer to malloc'd storage, which
+ * must be freed by the caller (m_free).  Taken from libiberty as
+ * found in gcc-2.95.2 and a little bit modernized.
+ * FIXME: Write a new CRT for W32.
+ */
+int
+vasprintf ( char **result, const char *format, va_list args)
+{
+  const char *p = format;
+  /* Add one to make sure that it is never zero, which might cause malloc
+     to return NULL.  */
+  int total_width = strlen (format) + 1;
+  va_list ap;
+
+  /* this is not really portable but works under Windows */
+  memcpy ( &ap, &args, sizeof (va_list));
+
+  while (*p != '\0')
+    {
+      if (*p++ == '%')
+	{
+	  while (strchr ("-+ #0", *p))
+	    ++p;
+	  if (*p == '*')
+	    {
+	      ++p;
+	      total_width += abs (va_arg (ap, int));
+	    }
+	  else
+            {
+              char *endp;  
+              total_width += strtoul (p, &endp, 10);
+              p = endp;
+            }
+	  if (*p == '.')
+	    {
+	      ++p;
+	      if (*p == '*')
+		{
+		  ++p;
+		  total_width += abs (va_arg (ap, int));
+		}
+	      else
+                {
+                  char *endp;
+                  total_width += strtoul (p, &endp, 10);
+                  p = endp;
+                }
+	    }
+	  while (strchr ("hlL", *p))
+	    ++p;
+	  /* Should be big enough for any format specifier except %s
+             and floats.  */
+	  total_width += 30;
+	  switch (*p)
+	    {
+	    case 'd':
+	    case 'i':
+	    case 'o':
+	    case 'u':
+	    case 'x':
+	    case 'X':
+	    case 'c':
+	      (void) va_arg (ap, int);
+	      break;
+	    case 'f':
+	    case 'e':
+	    case 'E':
+	    case 'g':
+	    case 'G':
+	      (void) va_arg (ap, double);
+	      /* Since an ieee double can have an exponent of 307, we'll
+		 make the buffer wide enough to cover the gross case. */
+	      total_width += 307;
+	    
+	    case 's':
+	      total_width += strlen (va_arg (ap, char *));
+	      break;
+	    case 'p':
+	    case 'n':
+	      (void) va_arg (ap, char *);
+	      break;
+	    }
+	}
+    }
+  *result = m_alloc (total_width);
+  if (*result != NULL)
+    return vsprintf (*result, format, args);
+  else
+    return 0;
+}
+
+#endif /*__MINGW32__*/
+
+
